#!/usr/bin/python
# -*- coding: utf-8 -*-

from cow.server import Server
from cow.plugins.sqlalchemy_plugin import SQLAlchemyPlugin
from cow.plugins.redis_plugin import RedisPlugin
from tornado.httpclient import AsyncHTTPClient
import tornado.locale
import redis
from materialgirl import Materializer
from materialgirl.storage.redis import RedisStorage

from holmes.handlers.auth import AuthenticateHandler
from holmes.handlers.page import (
    PageHandler, PageReviewsHandler, PageViolationsPerDayHandler, NextJobHandler
)
from holmes.handlers.violation import (
    MostCommonViolationsHandler, ViolationsHandler, ViolationHandler, ViolationDomainsHandler
)
from holmes.handlers.review import (
    ReviewHandler, LastReviewsHandler, ReviewsInLastHourHandler
)
from holmes.handlers.domains import (
    DomainsHandler, DomainDetailsHandler, DomainViolationsPerDayHandler,
    DomainReviewsHandler, DomainsChangeStatusHandler, DomainsFullDataHandler,
    DomainGroupedViolationsHandler, DomainTopCategoryViolationsHandler
)
from holmes.handlers.search import (
    SearchHandler
)
from holmes.handlers.request import (
    RequestDomainHandler, LastRequestsHandler, FailedResponsesHandler,
    LastRequestsStatusCodeHandler
)
from holmes.handlers.limiter import LimiterHandler
from holmes.handlers.domains_violations_prefs import (
    DomainsViolationsPrefsHandler
)
from holmes.handlers.users_violations_prefs import UsersViolationsPrefsHandler
from holmes.handlers.users import UserLocaleHandler

from holmes.handlers.bus import EventBusHandler
from holmes.event_bus import EventBus
from holmes.utils import load_classes, load_languages, locale_path
from holmes.models import Key, DomainsViolationsPrefs
from holmes.cache import Cache
from holmes import __version__
from holmes.handlers import BaseHandler


def main():
    AsyncHTTPClient.configure("tornado.curl_httpclient.CurlAsyncHTTPClient")
    HolmesApiServer.run()


class VersionHandler(BaseHandler):
    def get(self):
        self.write(__version__)


class HolmesApiServer(Server):
    def __init__(self, db=None, debug=None, *args, **kw):
        super(HolmesApiServer, self).__init__(*args, **kw)

        self.force_debug = debug
        self.db = db

    def get_extra_server_parameters(self):
        return {
            'no_keep_alive': False
        }

    def initialize_app(self, *args, **kw):
        super(HolmesApiServer, self).initialize_app(*args, **kw)

        self.application.db = None

        if self.force_debug is not None:
            self.debug = self.force_debug

    def get_handlers(self):
        uuid_regex = r'[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}'
        domain_regex = r'[a-z0-9]+(?:[\-\.]{1}[a-z0-9]+)*\.[a-z]{2,6}'
        key_name_regex = r'[_a-z0-9\.]+'
        numbers_regex = r'[0-9]+'

        handlers = [
            ('/most-common-violations/?', MostCommonViolationsHandler),
            ('/last-reviews/?', LastReviewsHandler),
            ('/reviews-in-last-hour/?', ReviewsInLastHourHandler, dict(is_public=True)),
            ('/page/(%s)/review/(%s)/?' % (uuid_regex, uuid_regex), ReviewHandler),
            ('/page/(%s)/reviews/?' % uuid_regex, PageReviewsHandler),
            ('/page/(%s)/violations-per-day/?' % uuid_regex, PageViolationsPerDayHandler),
            ('/page/(%s)/?' % uuid_regex, PageHandler),
            ('/search/?', SearchHandler),
            ('/page/?', PageHandler),
            ('/domains/?', DomainsHandler),
            ('/domains-details/?', DomainsFullDataHandler),
            ('/domains/(%s)/?' % domain_regex, DomainDetailsHandler),
            ('/domains/(%s)/violations-per-day/?' % domain_regex, DomainViolationsPerDayHandler),
            ('/domains/(%s)/violations-prefs/?' % domain_regex, DomainsViolationsPrefsHandler),
            ('/domains/(%s)/violations/?' % domain_regex, DomainGroupedViolationsHandler),
            ('/domains/(%s)/violations/(%s)/?' % (domain_regex, numbers_regex), DomainTopCategoryViolationsHandler),
            ('/domains/(%s)/reviews/?' % domain_regex, DomainReviewsHandler),
            ('/domains/(%s)/change-status/?' % domain_regex, DomainsChangeStatusHandler),
            ('/domains/(%s)/requests/(%s)/?' % (domain_regex, numbers_regex), RequestDomainHandler),
            ('/events/?', EventBusHandler),
            ('/violations/?', ViolationsHandler),
            ('/violation/(%s)/?' % key_name_regex, ViolationHandler),
            ('/violation/(%s)/domains/?' % key_name_regex, ViolationDomainsHandler),
            ('/limiters/?', LimiterHandler),
            ('/limiters/(%s)/?' % numbers_regex, LimiterHandler),
            ('/next-jobs/?', NextJobHandler),
            ('/last-requests/?', LastRequestsHandler),
            ('/last-requests/status-code/?', LastRequestsStatusCodeHandler),
            ('/last-requests/failed-responses/?', FailedResponsesHandler),
            ('/version/?', VersionHandler, dict(is_public=True)),
            ('/authenticate/?', AuthenticateHandler, dict(is_public=True)),
            ('/users/violations-prefs/?', UsersViolationsPrefsHandler),
            ('/users/locale/?', UserLocaleHandler),
        ]

        return tuple(handlers)

    def get_plugins(self):
        return [
            SQLAlchemyPlugin,
            RedisPlugin
        ]

    def after_start(self, io_loop):
        if self.db is not None:
            self.application.db = self.db
        else:
            self.application.db = self.application.get_sqlalchemy_session()

        if self.debug:
            from sqltap import sqltap
            self.sqltap = sqltap.start()

        authnz_wrapper_class = self._load_authnz_wrapper()
        if authnz_wrapper_class:
            self.application.authnz_wrapper = authnz_wrapper_class(self.application.config)
        else:
            self.application.authnz_wrapper = None

        self.application.facters = self._load_facters()
        self.application.validators = self._load_validators()
        self.application.error_handlers = [handler(self.application.config) for handler in self._load_error_handlers()]

        self.application.search_provider = self._load_search_provider()(
            config=self.application.config,
            db=self.application.db,
            authnz_wrapper=self.application.authnz_wrapper,
            io_loop=io_loop
        )

        self.application.fact_definitions = {}
        self.application.violation_definitions = {}

        self.application.default_violations_values = {}

        for facter in self.application.facters:
            self.application.fact_definitions.update(facter.get_fact_definitions())

        Key.insert_keys(self.application.db, self.application.fact_definitions)

        for validator in self.application.validators:
            self.application.violation_definitions.update(validator.get_violation_definitions())

            self.application.default_violations_values.update(
                validator.get_default_violations_values(self.application.config)
            )

        Key.insert_keys(
            self.application.db,
            self.application.violation_definitions,
            self.application.default_violations_values
        )

        self.application.event_bus = EventBus(self.application)
        self.application.http_client = AsyncHTTPClient(io_loop=io_loop)

        self.application.cache = Cache(self.application)

        self.configure_material_girl()

        self.configure_i18n()

        DomainsViolationsPrefs.insert_default_violations_values_for_all_domains(
            self.application.db,
            self.application.default_violations_values,
            self.application.violation_definitions,
            self.application.cache
        )

    def configure_material_girl(self):
        from holmes.material import configure_materials

        host = self.config.get('MATERIAL_GIRL_REDISHOST')
        port = self.config.get('MATERIAL_GIRL_REDISPORT')

        self.redis_material = redis.StrictRedis(host=host, port=port, db=0)

        self.application.girl = Materializer(
            storage=RedisStorage(redis=self.redis_material),
            load_on_cachemiss=self.config.get('MATERIAL_GIRL_LOAD_ON_CACHEMISS', True)
        )

        configure_materials(self.application.girl, self.application.db, self.config)

    def _load_validators(self):
        return load_classes(default=self.config.VALIDATORS)

    def _load_facters(self):
        return load_classes(default=self.config.FACTERS)

    def _load_error_handlers(self):
        return load_classes(default=self.config.ERROR_HANDLERS)

    def _load_authnz_wrapper(self):
        authnz_wrapper_class_name = self.config.get('AUTHNZ_WRAPPER', None)
        if authnz_wrapper_class_name:
            authnz_wrapper_list = load_classes(default=[authnz_wrapper_class_name])
            if isinstance(authnz_wrapper_list, list) and len(authnz_wrapper_list) == 1:
                return authnz_wrapper_list.pop()
        return None

    def _load_search_provider(self):
        search_provider = load_classes(default=[self.config.SEARCH_PROVIDER])
        if isinstance(search_provider, list) and len(search_provider) == 1:
            return search_provider.pop()
        else:
            raise Exception('A search provider must be defined!')

    def before_end(self, io_loop):
        self.application.db.remove()

        if self.debug and getattr(self, 'sqltap', None) is not None:
            from sqltap import sqltap

            statistics = self.sqltap.collect()
            sqltap.report(statistics, "report.html")

    def configure_i18n(self):
        load_languages()
        tornado.locale.load_gettext_translations(locale_path, 'api')

if __name__ == '__main__':
    main()



"""change current_review column in worker table to current_url

Revision ID: 2a535a54aa95
Revises: f851ec9af09
Create Date: 2013-11-18 15:05:07.519054

"""

# revision identifiers, used by Alembic.
revision = '2a535a54aa95'
down_revision = 'f851ec9af09'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column(
        'workers',
        sa.Column('current_url', sa.Text)
    )

    op.drop_constraint('fk_worker_current_review', 'workers', type_="foreignkey")
    op.drop_column('workers', 'current_review_id')


def downgrade():
    op.add_column(
        'workers',
        sa.Column('current_review_id', sa.Integer)
    )

    op.create_foreign_key(
        "fk_worker_current_review", "workers",
        "reviews", ["current_review_id"], ["id"]
    )

    op.drop_column('workers', 'current_url')



"""add_index_to_violations_id_and_key_id

Revision ID: 1e071953a65d
Revises: 2f32c106a437
Create Date: 2013-12-05 23:53:53.303743

"""

# revision identifiers, used by Alembic.
revision = '1e071953a65d'
down_revision = '2f32c106a437'

from alembic import op


def upgrade():
    op.create_index('idx_violations_id_key_id', 'violations', ['key_id', 'id'])


def downgrade():
    op.drop_constraint('fk_key_violation', 'violations', type_='foreignkey')
    op.drop_index('idx_violations_id_key_id', 'violations')
    op.create_foreign_key(
        'fk_key_violation', 'violations',
        'keys', ['key_id'], ["id"]
    )



#!/usr/bin/python
# -*- coding: utf-8 -*-

from holmes.facters import Facter
from holmes.utils import _


class HeadingHierarchyFacter(Facter):
    @classmethod
    def get_fact_definitions(cls):
        return {
            'page.heading_hierarchy': {
                'title': _('Heading Hierarchy'),
                'description': lambda value: list(value),
                'unit': 'heading-hierarchy',
                'category': _('SEO'),
            },
        }

    def get_facts(self):
        elements = self.get_heading()

        self.review.data['page.heading_hierarchy'] = []

        for element in elements:
            data = (element.tag, element.text_content().strip())
            self.review.data['page.heading_hierarchy'].append(data)

        if self.review.data['page.heading_hierarchy']:
            self.add_fact(
                key='page.heading_hierarchy',
                value=self.review.data['page.heading_hierarchy']
            )

    def get_heading(self):
        return self.reviewer.current_html.cssselect('body h1,h2,h3,h4,h5,h6')



#!/usr/bin/python
# -*- coding: utf-8 -*-

from holmes.validators.base import Validator
from holmes.utils import _


class SchemaOrgItemTypeValidator(Validator):
    @classmethod
    def get_violation_definitions(cls):
        return {
            'absent.schema.itemscope': {
                'title': _('itemscope attribute missing in body'),
                'description': _(
                    'In order to conform to schema.org definition '
                    'of a webpage, the body tag must feature an '
                    'itemscope attribute.'),
                'category': _('SEO'),
                # TODO: make a better generic_description, more descritive
                'generic_description': _(
                    'In order to conform to schema.org definition '
                    'of a webpage, the body tag must feature an '
                    'itemscope attribute.')
            },
            'absent.schema.itemtype': {
                'title': _('itemtype attribute missing in body'),
                'description': _(
                    'In order to conform to schema.org definition '
                    'of a webpage, the body tag must feature an '
                    'itemtype attribute.'),
                'category': _('SEO'),
                # TODO: make a better generic_description, more descritive
                'generic_description': _(
                    'In order to conform to schema.org definition '
                    'of a webpage, the body tag must feature an '
                    'itemtype attribute.')
            },
            'invalid.schema.itemtype': {
                'title': _('itemtype attribute is invalid'),
                'description': _(
                    'In order to conform to schema.org definition '
                    'of a webpage, the body tag must feature an '
                    'itemtype attribute with a value of '
                    '"http://schema.org/WebPage" or '
                    'one of its more specific types.'),
                'category': _('SEO'),
                # TODO: make a better generic_description, more descritive
                'generic_description': _(
                    'In order to conform to schema.org definition '
                    'of a webpage, the body tag must feature an '
                    'itemtype attribute with a value of '
                    '"http://schema.org/WebPage" or '
                    'one of its more specific types.'),
                'unit': 'list'
            }
        }

    @classmethod
    def get_default_violations_values(cls, config):
        return {
            'invalid.schema.itemtype': {
                'value': config.SCHEMA_ORG_ITEMTYPE,
                'description': config.get_description('SCHEMA_ORG_ITEMTYPE')
            }
        }

    def validate(self):
        itemtype_value = self.get_violation_pref('invalid.schema.itemtype')

        body = self.get_body()

        if not body:
            return

        if len(body) == 1:

            body = body[0]
            attributes = body.keys()

            if 'itemscope' not in attributes:
                self.add_violation(
                    key='absent.schema.itemscope',
                    value=None,
                    points=10)

            has_itemtype = 'itemtype' in attributes
            if not has_itemtype:
                self.add_violation(
                    key='absent.schema.itemtype',
                    value=None,
                    points=10
                )

            if has_itemtype and body.get('itemtype') not in itemtype_value:
                self.add_violation(
                    key='invalid.schema.itemtype',
                    value=None,
                    points=10
                )

    def get_body(self):
        return self.review.data.get('page.body', None)



"""Converting the value of violations and facts to BLOB

Revision ID: 155c6ce689ed
Revises: 1c9004f0ab21
Create Date: 2014-04-03 09:03:49.118969

"""

# revision identifiers, used by Alembic.
revision = '155c6ce689ed'
down_revision = '1c9004f0ab21'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.alter_column('facts', 'value', type_=sa.BLOB(length=4294967295), nullable=False)
    op.alter_column('violations', 'value', type_=sa.BLOB(length=4294967295), nullable=False)


def downgrade():
    op.alter_column('facts', 'value', type_=sa.Text(length=4294967295), nullable=False)
    op.alter_column('violations', 'value', type_=sa.Text(length=4294967295), nullable=False)



#!/usr/bin/python
# -*- coding: utf-8 -*-

from tornado import gen
from functools import partial

from holmes.handlers import BaseHandler
from holmes.models import Review, Domain


class MostCommonViolationsHandler(BaseHandler):

    @gen.coroutine
    def get(self):
        violations = dict(self.girl.get('most_common_violations') or [])

        result = []
        for violation in self.application.violation_definitions.values():
            result.append({
                'name': self._(violation['title']),
                'key': violation['key'].name,
                'category': self._(violation['category']),
                'count': violations.get(violation['key'].name, 0),
            })

        self.write_json(result)
        self.finish()


class ViolationHandler(BaseHandler):

    @gen.coroutine
    def get(self, key_name):
        current_page = int(self.get_argument('current_page', 1))
        page_size = int(self.get_argument('page_size', 10))
        domain_filter = self.get_argument('domain_filter', None)
        page_filter = self.get_argument('page_filter', None)

        domain = None
        if domain_filter is not None:
            domain = Domain.get_domain_by_name(domain_filter, self.db)
            if not domain:
                self.set_status(404, self._('Domain %s not found') % domain_filter)
                self.finish()
                return

        violations = self.application.violation_definitions
        if key_name not in violations:
            self.set_status(404, self._('Invalid violation key %s') % key_name)
            self.finish()
            return

        violation_title = violations[key_name]['title']
        key_id = violations[key_name]['key'].id

        violation = yield self.application.search_provider.get_by_violation_key_name(
            key_id=key_id,
            current_page=current_page,
            page_size=page_size,
            domain=domain,
            page_filter=page_filter,
        )

        if 'error' in violation:
            self.set_status(violation['error']['status_code'], violation['error']['reason'])
            self.finish()
            return

        if 'reviewsCount' not in violation:
            if not domain and not page_filter:
                violation['reviewsCount'] = Review.count_by_violation_key_name(self.db, key_id)
            else:
                violation['reviewsCount'] = None

        violation['title'] = violation_title

        self.write_json(violation)
        self.finish()


class ViolationDomainsHandler(BaseHandler):

    def __init__(self, *args, **kw):
        super(ViolationDomainsHandler, self).__init__(*args, **kw)
        self.key_details_handler = {
            'blacklist.domains': partial(self.girl.get, 'blacklist_domain_count')
        }

    @gen.coroutine
    def get(self, key_name):
        violations = self.application.violation_definitions

        if key_name not in violations:
            self.set_status(404, self._('Invalid violation key %s') % key_name)
            return

        violation_title = violations[key_name]['title']
        violation_description = violations[key_name]['generic_description']
        violation_category = violations[key_name]['category']
        key_id = violations[key_name]['key'].id

        grouped_violations = self.girl.get('violation_count_for_domains') or {}
        domains = grouped_violations.get(key_id, [])

        violation = {
            'title': self._(violation_title),
            'description': self._(violation_description),
            'category': self._(violation_category),
            'domains': domains,
            'total': sum(domain['count'] for domain in domains)
        }

        if key_name in self.key_details_handler:
            violation['details'] = self.key_details_handler[key_name]() or []

        self.write_json(violation)
        self.finish()


class ViolationsHandler(BaseHandler):
    def get(self):
        violations = self.application.violation_definitions

        json = []
        for key in violations.keys():
            violation = violations.get(key)
            json.append({
                'key_name': key,
                'title': self._(violation.get('title')),
                'category': self._(violation.get('category', None))
            })

        self.write_json(json)
        self.finish()



"""added domain_id in violations table

Revision ID: 49d09b3d2801
Revises: 43ce6999f48e
Create Date: 2014-02-20 15:33:25.850731

"""

# revision identifiers, used by Alembic.
revision = '49d09b3d2801'
down_revision = '43ce6999f48e'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('violations', sa.Column('domain_id', sa.Integer, nullable=True))

    connection = op.get_bind()
    connection.execute('UPDATE violations AS viol SET viol.domain_id = (SELECT rev.domain_id FROM reviews AS rev WHERE rev.id = viol.review_id)')

    op.alter_column('violations', 'domain_id', type_=sa.Integer, nullable=False)

    op.create_foreign_key(
        'fk_violation_domain', 'violations',
        'domains', ['domain_id'], ['id']
    )


def downgrade():
    op.drop_constraint('fk_violation_domain', 'violations', type_='foreignkey')
    op.drop_column('violations', 'domain_id')



#!/usr/bin/python
# -*- coding: utf-8 -*-

from holmes.validators.base import Validator
from holmes.utils import _


class HeadingHierarchyValidator(Validator):

    @classmethod
    def get_violation_parsed_value(cls, value):
        return {
            'max_size': value['max_size'],
            'hh_list': ''.join([
                '<li><span class="hh-type">%s</span>: %s</li>' % (key, desc)
                for key, desc in value['hh_list']
            ])
        }

    @classmethod
    def get_violation_definitions(cls):
        return {
            'page.heading_hierarchy.size': {
                'title': _('Maximum size of a heading hierarchy'),
                'description': _(
                    'Heading hierarchy values bigger than %(max_size)s '
                    'characters aren\'t good for Search Engines. This '
                    'elements were found: '
                    '<ul class="violation-hh-list">%(hh_list)s</ul>'),
                'value_parser': cls.get_violation_parsed_value,
                'category': _('SEO'),
                'generic_description': _(
                    'Heading hierarchy tags with values too big aren\'t good '
                    'for search engines. The definition of this maximum size '
                    'can be configurable on Holmes.'
                ),
                'unit': 'number'
            }
        }

    @classmethod
    def get_default_violations_values(cls, config):
        return {
            'page.heading_hierarchy.size': {
                'value': config.MAX_HEADING_HIEARARCHY_SIZE,
                'description': config.get_description('MAX_HEADING_HIEARARCHY_SIZE')
            }
        }

    def validate(self):
        max_heading_size = self.get_violation_pref('page.heading_hierarchy.size')

        hh_list = self.review.data.get('page.heading_hierarchy', [])

        violations = []
        for key, desc in hh_list:
            if len(desc) > max_heading_size:
                violations.append((key, desc))

        if violations:
            self.add_violation(
                key='page.heading_hierarchy.size',
                value={
                    'max_size': max_heading_size,
                    'hh_list': violations
                },
                points=20 * len(violations)
            )


class H1HeadingValidator(Validator):
    @classmethod
    def get_violation_definitions(cls):
        return {
            'page.heading_hierarchy.h1_not_found': {
                'title': _('H1 Headings not found'),
                'description': _(
                    'Your page does not contain any H1 headings. H1 headings '
                    'help indicate the important topics of your page to '
                    'search engines. While less important than good '
                    'meta-titles and descriptions, H1 headings may still '
                    'help define the topic of your page to search engines.'
                ),
                'category': 'SEO',
                'generic_description': (
                    'H1 headings help indicate the important topics of your '
                    'page to search engines. While less important than good '
                    'meta-titles and descriptions, H1 headings may still '
                    'help define the topic of your page to search engines.'
                )
            }
        }

    def validate(self):
        hh_list = self.review.data.get('page.heading_hierarchy', [])
        for key, desc in hh_list:
            if key == 'h1':
                return

        self.add_violation(
            key='page.heading_hierarchy.h1_not_found',
            value=None,
            points=30
        )



# -*- coding: utf-8 -*-

from datetime import datetime, timedelta
from ujson import loads

from tornado import gen
from tornado.httpclient import HTTPError

from holmes.models import User
from holmes.handlers import BaseHandler


class AuthenticateHandler(BaseHandler):

    def get(self):
        '''
        Only returns true or false if is a valid authenticated request
        '''
        self.set_status(200)
        user = self.get_authenticated_user()
        if user:
            self.write_json({
                'authenticated': True, 'isSuperUser': user.is_superuser
            })
        else:
            self.write_json({
                'authenticated': False, 'isSuperUser': False
            })

    @gen.coroutine
    def post(self):
        '''
        Try to authenticate user with the provider and access_token POST data.
        If the `self.authenticate` method returns the user, create a JSON
        Web Token (JWT) and set a `HOLMES_AUTH_TOKEN` cookie with the encoded
        value. Otherwise returns a unauthorized request.
        '''
        post_data = loads(self.request.body)

        provider = post_data.get('provider')
        access_token = post_data.get('access_token')

        user = yield self.authenticate(provider, access_token)
        if user:

            payload = dict(
                sub=user.email,
                iss=user.provider,
                token=access_token,
                iat=datetime.utcnow(),
                exp=datetime.utcnow() + timedelta(
                    seconds=self.config.SESSION_EXPIRATION
                )
            )
            auth_token = self.jwt.encode(payload)

            self.set_cookie('HOLMES_AUTH_TOKEN', auth_token)
            self.write_json(dict(authenticated=True, first_login=user.first_login))
        else:
            self.set_unauthorized()

    def delete(self):
        self.clear_cookie('HOLMES_AUTH_TOKEN')
        self.write_json({'loggedOut': True})

    @gen.coroutine
    def authenticate(self, provider, access_token):
        '''
        Authenticate user with the given access_token on the specific
        provider method. If it returns the user data, try to fetch the user
        on the database or create user if it doesn`t exist and then return
        the user object. Otherwise, returns None, meaning invalid
        authentication parameters.
        '''

        if provider == u'GooglePlus':
            oauth_user = yield self.authenticate_on_google(access_token)
        else:
            oauth_user = None

        if oauth_user:
            db = self.application.db
            user = User.by_email(oauth_user['email'], db)
            if user:
                user.last_login = datetime.utcnow()
                db.flush()
                db.commit()  # FIXME, test if commit() is necessary
                user.first_login = False
            else:
                user = User.add_user(
                    db, oauth_user['fullname'], oauth_user['email'], provider,
                    datetime.utcnow()
                )
                user.first_login = True
        else:
            user = None

        raise gen.Return(user)

    @gen.coroutine
    def authenticate_on_google(self, access_token):
        '''
        Try to get Google user info and returns it if
        the given access_token get`s a valid user info in a string
        json format. If the response was not an status code 200 or
        get an error on Json, None was returned.

        Example of return on success:
        {
            id: "1234567890abcdef",
            email: "...@gmail.com",
            fullname: "Ricardo L. Dani",
        }
        '''

        response = yield self._fetch_google_userinfo(access_token)

        if response.code == 200:
            body = loads(response.body)
            if not body.get('error'):
                raise gen.Return({
                    'email': body.get("email"),
                    'fullname': body.get("name"),
                    'id': body.get("id")
                })

        raise gen.Return(None)

    @gen.coroutine
    def _fetch_google_userinfo(self, access_token):
        google_api_url = 'https://www.googleapis.com/oauth2/v1/userinfo'
        url = '%s?access_token=%s' % (google_api_url, access_token)

        try:
            response = yield self.application.http_client.fetch(
                url,
                proxy_host=self.application.config.HTTP_PROXY_HOST,
                proxy_port=self.application.config.HTTP_PROXY_PORT
            )
        except HTTPError, e:
            response = e.response

        raise gen.Return(response)



"""create AOPIC score and lambda page

Revision ID: 4779cd2391f7
Revises: 368929fa3075
Create Date: 2014-01-14 19:30:17.565805

"""

# revision identifiers, used by Alembic.
revision = '4779cd2391f7'
down_revision = '368929fa3075'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column(
        'pages',
        sa.Column('score', sa.Float, nullable=False, server_default=sa.text('0.0'))
    )

    op.create_table(
        'settings',
        sa.Column('id', sa.Integer, primary_key=True),
        sa.Column('lambda_score', sa.Float, server_default=sa.text('0.0'), nullable=False)
    )

    connection = op.get_bind()
    connection.execute('INSERT INTO settings(lambda_score) VALUES(0.0)')

    op.create_index('idx_pages_score', 'pages', ['score'])


def downgrade():
    op.drop_table('settings')

    op.drop_index('idx_pages_score', 'pages')
    op.drop_column('pages', 'score')



#!/usr/bin/python
# -*- coding: utf-8 -*-

from holmes.utils import get_domain_from_url, is_valid
from holmes.validators.base import Validator
from holmes.utils import _


class BlackListValidator(Validator):
    @classmethod
    def get_blacklist_parsed_value(cls, value):
        return ', '.join([
            '<a href="%s" target="_blank">Link #%s</a>' % (url, index)
            for index, url in enumerate(value)
        ])

    @classmethod
    def get_violation_definitions(cls):
        return {
            'blacklist.domains': {
                'title': _('Domain Blacklist'),
                'description': _('Some links are blacklisted: %s'),
                'value_parser': cls.get_blacklist_parsed_value,
                'category': _('SEO'),
                'generic_description': _(
                    'Detected domain blacklisted hyperlinks. '
                    'Links with this violation are those that have anchors '
                    'to websites added in Holmes\'s Black List configuration.'
                ),
                'unit': 'list'
            }
        }

    @classmethod
    def get_default_violations_values(cls, config):
        return {
            'blacklist.domains': {
                'value': config.BLACKLIST_DOMAIN,
                'description': config.get_description('BLACKLIST_DOMAIN')
            }
        }

    def validate(self):
        blacklist_domains = self.get_violation_pref('blacklist.domains')

        domains = []

        links = self.get_links()

        for link in links:
            href = link.get('href')

            if not is_valid(href):
                continue

            link_domain, link_domain_url = get_domain_from_url(href)
            if link_domain in blacklist_domains:
                domains.append(href)

        if domains:
            self.add_violation(
                key='blacklist.domains',
                value=domains,
                points=100 * len(domains)
            )

    def get_links(self):
        return self.review.data.get('page.all_links', None)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from tornado.concurrent import return_future


class SearchProvider(object):
    def __init__(self, config, db, authnz_wrapper=None, io_loop=None):
        raise NotImplementedError()

    def index_review(self, review):
        raise NotImplementedError()

    @return_future
    def get_by_violation_key_name(self, key_id, current_page=1, page_size=10, domain=None, page_filter=None, callback=None):
        raise NotImplementedError()

    @return_future
    def get_domain_active_reviews(self, domain, current_page=1, page_size=10, page_filter=None, callback=None):
        raise NotImplementedError()

    @classmethod
    def argparser(cls):
        import argparse

        parser = argparse.ArgumentParser(description='Setup Holmes index on an ElasticSearch server')
        parser.add_argument(
            '-c', '--conf',
            nargs=1,
            metavar='conf_file',
            help='path to configuration file'
        )
        parser.add_argument(
            '-s', '--server',
            nargs=1,
            metavar='host:port',
            help='elastic search server host and port'
        )
        parser.add_argument(
            '-i', '--index',
            nargs=1,
            metavar='index_name',
            help='name of the index'
        )
        parser.add_argument(
            '--create',
            action='store_true',
            help='create the index'
        )
        parser.add_argument(
            '--recreate',
            action='store_true',
            help='recreate the index (use with caution)'
        )
        parser.add_argument(
            '--delete',
            action='store_true',
            help='delete the index (use with caution)'
        )
        parser.add_argument(
            '-k', '--keys',
            nargs='+',
            metavar='key',
            help='index reviews with violation of such keys'
        )
        parser.add_argument(
            '-a', '--all-keys',
            action='store_true',
            help='index all reviews with at least one violation of any key (might take long)'
        )
        parser.add_argument(
            '-b', '--batch-size',
            type=int,
            nargs=1,
            metavar='N',
            help='batch size (default is 200)'
        )
        parser.add_argument(
            '-r', '--replace',
            action='store_true',
            help='replace entire index (default is increment/resume)'
        )
        parser.add_argument(
            '-v', '--verbose',
            action='count',
            default=0,
            help='log level: v=warning, vv=info, vvv=debug'
        )

        return parser

    @classmethod
    def new_instance(cls, config):
        raise NotImplementedError()

    @classmethod
    def main(cls):
        raise NotImplementedError()

if __name__ == '__main__':
    SearchProvider.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
from uuid import uuid4
from functools import partial
from collections import defaultdict

from holmes.cli import BaseCLI
from holmes.models.domain import Domain
from holmes.models.violation import Violation
from holmes.models.request import Request
from holmes.utils import get_domain_from_url


# Materials grouped by domain should be expired bellow, this function is called
# every time a new domain is added (see holmes/models/page.py:Page.add_domain)
def expire_materials(girl):
    girl.expire('domains_details')
    girl.expire('failed_responses_count')
    girl.expire('violation_count_for_domains')
    girl.expire('top_violations_in_category_for_domains')


def configure_materials(girl, db, config):
    girl.add_material(
        'domains_details',
        partial(Domain.get_domains_details, db),
        config.MATERIALS_EXPIRATION_IN_SECONDS['domains_details'],
        config.MATERIALS_GRACE_PERIOD_IN_SECONDS['domains_details'],
        config.MATERIALS_LOCK_TIMEOUT_IN_SECONDS['domains_details']
    )

    girl.add_material(
        'violation_count_for_domains',
        partial(MaterialConveyor.get_violation_count_for_domains, db),
        config.MATERIALS_EXPIRATION_IN_SECONDS['violation_count_for_domains'],
        config.MATERIALS_GRACE_PERIOD_IN_SECONDS['violation_count_for_domains'],
        config.MATERIALS_LOCK_TIMEOUT_IN_SECONDS['violation_count_for_domains']
    )

    girl.add_material(
        'violation_count_by_category_for_domains',
        partial(Violation.get_group_by_category_id_for_all_domains, db),
        config.MATERIALS_EXPIRATION_IN_SECONDS['violation_count_by_category_for_domains'],
        config.MATERIALS_GRACE_PERIOD_IN_SECONDS['violation_count_by_category_for_domains'],
        config.MATERIALS_LOCK_TIMEOUT_IN_SECONDS['violation_count_by_category_for_domains']
    )

    girl.add_material(
        'top_violations_in_category_for_domains',
        partial(
            MaterialConveyor.get_top_violations_in_category_for_all_domains,
            db,
            config.get('TOP_CATEGORY_VIOLATIONS_SAMPLE_LIMIT')
        ),
        config.MATERIALS_EXPIRATION_IN_SECONDS['top_violations_in_category_for_domains'],
        config.MATERIALS_GRACE_PERIOD_IN_SECONDS['top_violations_in_category_for_domains'],
        config.MATERIALS_LOCK_TIMEOUT_IN_SECONDS['top_violations_in_category_for_domains']
    )

    girl.add_material(
        'blacklist_domain_count',
        partial(MaterialConveyor.get_blacklist_domain_count, db),
        config.MATERIALS_EXPIRATION_IN_SECONDS['blacklist_domain_count'],
        config.MATERIALS_GRACE_PERIOD_IN_SECONDS['blacklist_domain_count'],
        config.MATERIALS_LOCK_TIMEOUT_IN_SECONDS['blacklist_domain_count']
    )

    girl.add_material(
        'most_common_violations',
        partial(
            Violation.get_most_common_violations_names,
            db,
            config.get('MOST_COMMON_VIOLATIONS_SAMPLE_LIMIT')
        ),
        config.MATERIALS_EXPIRATION_IN_SECONDS['most_common_violations'],
        config.MATERIALS_GRACE_PERIOD_IN_SECONDS['most_common_violations'],
        config.MATERIALS_LOCK_TIMEOUT_IN_SECONDS['most_common_violations']
    )

    girl.add_material(
        'failed_responses_count',
        partial(
            Request.get_requests_count_by_status,
            db,
            config.get('MAX_REQUESTS_FOR_FAILED_RESPONSES')
        ),
        config.MATERIALS_EXPIRATION_IN_SECONDS['failed_responses_count'],
        config.MATERIALS_GRACE_PERIOD_IN_SECONDS['failed_responses_count'],
        config.MATERIALS_LOCK_TIMEOUT_IN_SECONDS['failed_responses_count']
    )


class MaterialConveyor(object):
    @classmethod
    def get_blacklist_domain_count(cls, db):
        ungrouped = defaultdict(int)
        for urls, count in Violation.get_group_by_value_for_key(db, 'blacklist.domains'):
            for url in urls:
                domain, null = get_domain_from_url(url)
                ungrouped[domain] += count
        blacklist = sorted(ungrouped.items(), key=lambda xz: -xz[1])
        return [dict(zip(('domain', 'count'), x)) for x in blacklist]

    @classmethod
    def get_violation_count_for_domains(cls, db):
        grouped = {}

        violations_count = Violation.get_group_by_key_id_for_all_domains(db)

        for key_id, domain_name, count in violations_count:

            if key_id not in grouped:
                grouped[key_id] = []

            grouped[key_id].append({
                'name': domain_name,
                'count': count
            })

        return grouped

    @classmethod
    def get_top_violations_in_category_for_all_domains(cls, db, limit):
        grouped = {}

        top_violations = Violation.get_top_in_category_for_all_domains(db, limit)

        for domain_name, key_category_id, key_name, count in top_violations:

            if domain_name not in grouped:
                grouped[domain_name] = {}

            if key_category_id not in grouped[domain_name]:
                grouped[domain_name][key_category_id] = []

            grouped[domain_name][key_category_id].append({
                'key_name': key_name,
                'count': count
            })

        return grouped


class MaterialWorker(BaseCLI):
    def initialize(self):
        self.uuid = uuid4().hex

        self.error_handlers = [handler(self.config) for handler in self.load_error_handlers()]

        self.connect_sqlalchemy()
        self.connect_to_redis()

        self.configure_material_girl()

    def do_work(self):
        self.info('Running material girl...')
        self.db.begin(subtransactions=True)
        self.girl.run()
        self.db.close()


def main():
    worker = MaterialWorker(sys.argv[1:])
    worker.run()

if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

import email.utils as eut
from datetime import datetime

from holmes.facters import Facter
from holmes.utils import _


class LastModifiedFacter(Facter):

    @classmethod
    def get_fact_definitions(cls):
        return {
            'page.last_modified': {
                'title': _('Last-Modified'),
                'description': lambda value: value,
                'category': _('HTTP'),
                'unit': 'datetime'
            }
        }

    def get_facts(self):
        last_modified = self.get_last_modified()

        if not last_modified:
            return

        self.review.data['page.last_modified'] = last_modified
        self.add_fact(key='page.last_modified', value=last_modified)

    def get_last_modified(self):
        last_modified = None

        headers = self.reviewer.current.headers

        modified = headers.get('Last-Modified', None)
        if modified:
            lt = eut.parsedate(modified)
            if lt is not None:
                last_modified = datetime(*lt[:6])

        return last_modified



#!/usr/bin/python
# -*- coding: utf-8 -*-

import logging
from uuid import uuid4

import holmes.utils as utils
from tornado.websocket import WebSocketHandler


class EventBusHandler(WebSocketHandler):

    @property
    def config(self):
        return self.application.config

    def initialize(self, *args, **kw):
        super(EventBusHandler, self).initialize(*args, **kw)

        self.jwt = utils.Jwt(self.config.SECRET_KEY)

    def keep_alive(self):
        return True

    def is_authenticated(self):
        return self.jwt.try_to_decode(self.get_cookie('HOLMES_AUTH_TOKEN'))

    def open(self):
        if self.is_authenticated()[0]:
            self.uuid = uuid4()
            logging.debug("WebSocket opened.")
            self.application.event_bus.subscribe(
                'events', self.uuid, self.async_callback(self.on_event)
            )
        else:
            self.close()

    def on_event(self, message):
        if self.ws_connection is not None:
            self.write_message(message)

    def on_close(self):
        if hasattr(self, 'uuid'):
            self.application.event_bus.unsubscribe('events', self.uuid)
        logging.debug("WebSocket closed.")



#!/usr/bin/python
# -*- coding: utf-8 -*-

from holmes.validators.base import Validator
from holmes.utils import _


class LastModifiedValidator(Validator):

    @classmethod
    def get_violation_definitions(cls):
        return {
            'page.last_modified.not_found': {
                'title': _('Last-Modified not found'),
                'description': _('Last-Modified was not found on %s.'),
                'category': _('HTTP'),
                'generic_description': _(
                    'The Last-Modified Header information allows Web Crawlers '
                    '(like Holmes) and search engines to know if the page is '
                    'modified since the last content retrival. If '
                    'Last-Modified wasn\'t present, all requests made by '
                    'this engines on the page will proccess all the content '
                    'and can be performatically bad to the server.'
                )
            }
        }

    def validate(self):
        last_modified = self.get_last_modified()

        if not last_modified:
            page_url = self.reviewer.page_url

            self.add_violation(
                key='page.last_modified.not_found',
                value=page_url,
                points=50
            )

    def get_last_modified(self):
        return self.review.data.get('page.last_modified', None)



"""create completed_day column

Revision ID: 2f32c106a437
Revises: 4cf3e398a81f
Create Date: 2013-12-05 14:09:40.756413

"""

# revision identifiers, used by Alembic.
revision = '2f32c106a437'
down_revision = '4cf3e398a81f'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('reviews', sa.Column('completed_day', sa.Date, nullable=True))

    connection = op.get_bind()
    connection.execute('UPDATE reviews set completed_day=DATE(completed_date) where is_complete=1;')


def downgrade():
    op.drop_column('reviews', 'completed_day')



"""add last_modified and expires to pages

Revision ID: 39091f5a3e5f
Revises: 4ff6c7ad84c6
Create Date: 2013-11-28 23:42:57.692382

"""

# revision identifiers, used by Alembic.
revision = '39091f5a3e5f'
down_revision = '4ff6c7ad84c6'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column(
        'pages',
        sa.Column('last_modified', sa.DateTime, nullable=True)
    )

    op.add_column(
        'pages',
        sa.Column('expires', sa.DateTime, nullable=True)
    )


def downgrade():
    op.drop_column('pages', 'last_modified')
    op.drop_column('pages', 'expires')



#!/usr/bin/python
# -*- coding: utf-8 -*-

from tornado.gen import coroutine

from holmes.utils import get_status_code_title
from holmes.models import Request
from holmes.handlers import BaseHandler


class RequestDomainHandler(BaseHandler):

    @coroutine
    def get(self, domain_name, status_code):

        if not domain_name:
            self.set_status(404, self._('Domain %s not found') % domain_name)
            return

        if not status_code:
            self.set_status(404, self._('Status code %s not found') % status_code)
            return

        if status_code == '200':
            self.set_status(403, self._('Status code %s is not allowed') % status_code)
            return

        current_page = int(self.get_argument('current_page', 1))
        page_size = int(self.get_argument('page_size', 10))

        requests = Request.get_requests_by_status_code(
            domain_name,
            status_code,
            self.db,
            current_page=current_page,
            page_size=page_size
        )

        requests_count = Request.get_requests_by_status_count(
            domain_name,
            status_code,
            self.db
        )

        result = {
            'statusCodeTitle': get_status_code_title(status_code),
            'requestsCount': requests_count,
            'requests': []
        }

        for request in requests:
            result['requests'].append({
                'id': request.id,
                'url': request.url,
                'review_url': request.review_url,
                'completed_date': request.completed_date
            })

        self.write_json(result)


class LastRequestsHandler(BaseHandler):
    @coroutine
    def get(self):
        requests = Request.get_last_requests(
            self.db,
            current_page=int(self.get_argument('current_page', 1)),
            page_size=int(self.get_argument('page_size', 10)),
            domain_filter=self.get_argument('domain_filter', None),
            status_code_filter=self.get_argument('status_code_filter', None)
        )

        self.write_json(
            {'requests': [request.to_dict() for request in requests]}
        )


class FailedResponsesHandler(BaseHandler):
    @coroutine
    def get(self):
        requests = self.girl.get('failed_responses_count') or []
        domain_filter = self.get_argument('domain_filter', None)

        if not domain_filter:
            domain_filter = '_all'

        result = []
        if domain_filter in requests:
            for status_code, count in requests[domain_filter]:
                result.append({
                    'statusCode': status_code,
                    'statusCodeTitle': get_status_code_title(status_code),
                    'count': count,
                })

        self.write_json(result)


class LastRequestsStatusCodeHandler(BaseHandler):
    @coroutine
    def get(self):
        status_code = Request.get_all_status_code(self.db)

        self.write_json(status_code)



#!/usr/bin/python
# -*- coding: utf-8 -*-


from holmes.validators.base import Validator
from holmes.utils import _


class RobotsValidator(Validator):

    SITEMAP_NOT_FOUND = _('You must specify the location of the Sitemap '
                          'using a robots.txt file')

    DISALLOW_NOT_FOUND = _('Disallow directive indicates that robots should '
                           'not access the specific directory, subdirectory '
                           'or file. ')

    DISALLOW_ROOT_PATH = _('Crawlers may not index anything, because '
                           'the root path (/) is disallowed.')

    @classmethod
    def get_violation_definitions(cls):
        return {
            'robots.not_found': {
                'title': _('Robots file not found'),
                'description': _("The robots file at '%s' was not found."),
                'category': _('SEO'),
                'generic_description': _(
                    'Validates the presence of robots file. A robots.txt file tells a search '
                    'engine what content they will index or not.'
                )
            },
            'robots.empty': {
                'title': _('Robots file was empty'),
                'description': _("The robots file at '%s' was empty."),
                'category': _('SEO'),
                'generic_description': _(
                    'Validates the content of a robots file. If empty, the search engines '
                    'will understand that has no robots file present.'
                )
            },
            'robots.sitemap.not_found': {
                'title': _('Sitemap in Robots not found'),
                'description': cls.SITEMAP_NOT_FOUND,
                'category': _('SEO'),
                'generic_description': _(
                    'Validates the presence of Sitemap in Robots.txt. '
                    'The Sitemap tells the robot how your website is '
                    'organized and witch way can be indexed.'
                )
            },
            'robots.disallow.not_found': {
                'title': _('Disallow in Robots not found'),
                'description': cls.DISALLOW_NOT_FOUND,
                'category': _('SEO'),
                'generic_description': _(
                    'Validates the presence of Disallow in Robots.txt. '
                    'Disallow lists the files and directories to be '
                    'excluded from indexing'
                )
            },
            'robots.disallow.root_path': {
                'title': _('Disallow: / in Robots'),
                'description': cls.DISALLOW_ROOT_PATH,
                'category': _('SEO'),
                'generic_description': _(
                    'Validates if the root path of your site is in '
                    'Disallow directive of Robots.txt. If true, the '
                    'root of your site will not be indexed on search '
                    'engines.'
                )
            },
        }

    def validate(self):
        if not self.reviewer.is_root():
            return

        response = self.review.data['robots.response']

        if response.status_code > 399:
            self.add_violation(
                key='robots.not_found',
                value=response.url,
                points=100
            )
            return

        if not response.text.strip():
            self.add_violation(
                key='robots.empty',
                value=response.url,
                points=100
            )
            return

        has_sitemap = False

        has_disallow = False
        disallow_root_path = False

        for rawline in response.text.splitlines():
            line = rawline.strip()
            comments = line.find('#')
            if comments >= 0:
                line = line[:comments]
            if line == '' or ':' not in line:
                continue
            key, val = [x.strip() for x in line.split(':', 1)]
            key = key.lower()
            if key == 'sitemap':
                has_sitemap = True
            elif key == 'disallow':
                has_disallow = True
                if val == '/':
                    disallow_root_path = True

        if not has_sitemap:
            self.add_violation(
                key='robots.sitemap.not_found',
                value=None,
                points=100
            )

        if not has_disallow:
            self.add_violation(
                key='robots.disallow.not_found',
                value=None,
                points=100
            )
        elif disallow_root_path:
            self.add_violation(
                key='robots.disallow.root_path',
                value=None,
                points=100
            )



"""Added User.provider column

Revision ID: 19ffab346000
Revises: 2620096d8dd2
Create Date: 2014-07-02 11:54:19.472151

"""

# revision identifiers, used by Alembic.
revision = '19ffab346000'
down_revision = '2620096d8dd2'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column(
        'users',
        sa.Column('provider', type_=sa.String(10), nullable=True)
    )


def downgrade():
    op.drop_column('users', 'provider')



#!/usr/bin/python
# -*- coding: utf-8 -*-

import sqlalchemy as sa
from sqlalchemy.orm import relationship
from sqlalchemy.orm.exc import NoResultFound

from holmes.models import Base


class KeysCategory(Base):
    __tablename__ = "keys_category"

    id = sa.Column(sa.Integer, primary_key=True)
    name = sa.Column('name', sa.String(255), nullable=False)

    keys = relationship("Key", cascade="all,delete", backref="category")

    def __str__(self):
        return '%s' % (self.name)

    def __repr__(self):
        return str(self)

    def to_dict(self):
        return {
            'name': self.name
        }

    @classmethod
    def get_by_id(cls, db, key_category_id):
        try:
            result = db \
                .query(KeysCategory) \
                .filter(KeysCategory.id == key_category_id) \
                .one()
        except NoResultFound:
            result = None

        return result

    @classmethod
    def get_by_name(cls, db, category_name):
        try:
            result = db \
                .query(KeysCategory) \
                .filter(KeysCategory.name == category_name) \
                .one()
        except NoResultFound:
            result = None

        return result

    @classmethod
    def get_or_create(cls, db, category_name):
        category = cls.get_by_name(db, category_name)

        if category is not None:
            return category

        query_params = {'name': category_name}

        db.execute(
            'INSERT INTO `keys_category` (name) ' \
            'VALUES (:name) ON DUPLICATE KEY ' \
            'UPDATE name = :name',
            query_params
        )

        return cls.get_by_name(db, category_name)

    @classmethod
    def insert_key_category(cls, db, key, name):
        category = KeysCategory.get_or_create(db, name)

        db.add(category)
        db.flush()
        db.commit()

        return category



"""Create DomainsViolationsPrefs table

Revision ID: 2620096d8dd2
Revises: 2901a4935e21
Create Date: 2014-05-23 15:42:29.172112

"""

# revision identifiers, used by Alembic.
revision = '2620096d8dd2'
down_revision = '2901a4935e21'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_table(
        'domains_violations_prefs',
        sa.Column('id', sa.Integer, primary_key=True),
        sa.Column('domain_id', sa.Integer, nullable=False),
        sa.Column('key_id', sa.Integer, nullable=False),
        sa.Column('value', sa.Text)
    )

    op.create_foreign_key(
        "fk_domain", "domains_violations_prefs",
        "domains", ["domain_id"], ["id"]
    )

    op.create_foreign_key(
        "fk_key", "domains_violations_prefs",
        "keys", ["key_id"], ["id"]
    )

    op.create_unique_constraint(
        'uk_domain_key',
        'domains_violations_prefs',
        ["key_id", 'domain_id']
    )


def downgrade():
    op.drop_constraint('fk_domain', 'domains_violations_prefs', type_="foreignkey")
    op.drop_constraint('fk_key', 'domains_violations_prefs', type_="foreignkey")
    op.drop_constraint('uk_domain_key', 'domains_violations_prefs', type_='unique')
    op.drop_table('domains_violations_prefs')



"""change value column type

Revision ID: f851ec9af09
Revises: 178c6db7516
Create Date: 2013-11-16 13:20:14.099934

"""

# revision identifiers, used by Alembic.
revision = 'f851ec9af09'
down_revision = '178c6db7516'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.alter_column('facts', 'value', type_=sa.Text, nullable=False)


def downgrade():
    op.alter_column('facts', 'value', type_=sa.String(4000), nullable=False)



"""Request: Added review_url column

Revision ID: 234e746c4c28
Revises: 22bc83869dbe
Create Date: 2014-01-27 14:41:11.003174

"""

# revision identifiers, used by Alembic.
revision = '234e746c4c28'
down_revision = '22bc83869dbe'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column(
        'requests',
        sa.Column('review_url', sa.Text, nullable=False)
    )


def downgrade():
    op.drop_column('requests', 'review_url')



"""Domain url index

Revision ID: 4260bbc20569
Revises: 4ab7a2e91f2d
Create Date: 2013-11-27 15:48:59.834199

"""

# revision identifiers, used by Alembic.
revision = '4260bbc20569'
down_revision = '4ab7a2e91f2d'

from alembic import op


def upgrade():
    op.execute('CREATE INDEX idx_domains_url on domains (url(220))')


def downgrade():
    op.drop_index('idx_domains_url', 'domains')



"""create violations category

Revision ID: 49c5cb27e7f6
Revises: 7fef2bae291
Create Date: 2014-01-21 19:46:14.259462

"""

# revision identifiers, used by Alembic.
revision = '49c5cb27e7f6'
down_revision = '7fef2bae291'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_table(
        'keys_category',
        sa.Column('id', sa.Integer, primary_key=True),
        sa.Column('name', sa.String(255), nullable=False),
    )

    op.add_column(
        'keys',
        sa.Column('category_id', sa.Integer),
    )

    op.create_foreign_key(
        "fk_keys_category", "keys",
        "keys_category", ["category_id"], ["id"]
    )


def downgrade():
    op.drop_constraint(
        'fk_keys_category',
        'keys',
        type_="foreignkey"
    )

    op.drop_column('keys', 'category_id')
    op.drop_table('keys_category')



#!/usr/bin/python
# -*- coding: utf-8 -*-

from holmes.validators.base import Validator
from holmes.utils import _


class ImageAltValidator(Validator):
    @classmethod
    def get_without_alt_parsed_value(cls, value):
        result = []
        for src, name in value:
            data = '<a href="%s" target="_blank">%s</a>' % (src, name)
            result.append(data)

        return {'images': ', '.join(result)}

    @classmethod
    def get_alt_too_big_parsed_value(cls, value):
        result = []
        for src, name, alt in value['images']:
            data = u'<a href="{}" alt="{}" target="_blank">{}</a>'.format(
                src, alt, name
            )
            result.append(data)

        return {
            'max_size': value['max_size'],
            'images': ', '.join(result)
        }

    @classmethod
    def get_violation_definitions(cls):
        return {
            'invalid.images.alt': {
                'title': _('Image(s) without alt attribute'),
                'description': _(
                    'Images without alt text are not good for '
                    'Search Engines. Images without alt were '
                    'found for: %(images)s.'),
                'value_parser': cls.get_without_alt_parsed_value,
                'category': _('SEO'),
                'generic_description': _(
                    'Images without alt attribute are not good for '
                    'search engines. They are searchable by the content '
                    'of this attribute, so if it\'s empty, it cause bad '
                    'indexing optimization.'
                )
            },
            'invalid.images.alt_too_big': {
                'title': _('Image(s) with alt attribute too big'),
                'description': _(
                    'Images with alt text bigger than %(max_size)d chars are '
                    'not good for search engines. Images with a too big alt '
                    'were found for: %(images)s.'),
                'value_parser': cls.get_alt_too_big_parsed_value,
                'category': _('SEO'),
                'generic_description': _(
                    'Images with alt text too long are not good to SEO. '
                    'This maximum value are configurable '
                    'by Holmes configuration.'
                ),
                'unit': 'number'
            }
        }

    @classmethod
    def get_default_violations_values(cls, config):
        return {
            'invalid.images.alt_too_big': {
                'value': config.MAX_IMAGE_ALT_SIZE,
                'description': config.get_description('MAX_IMAGE_ALT_SIZE')
            }
        }

    def validate(self):
        max_alt_size = self.get_violation_pref('invalid.images.alt_too_big')

        imgs = self.get_imgs()

        result_no_alt = []
        result_alt_too_big = []
        for img in imgs:
            src = img.get('src')
            if not src:
                continue

            src = self.normalize_url(src)
            img_alt = img.get('alt')

            if src:
                name = src.rsplit('/', 1)[-1]
                if not img_alt:
                    result_no_alt.append((src, name))
                elif len(img_alt) > max_alt_size:
                    result_alt_too_big.append((src, name, img_alt))

        if result_no_alt:
            self.add_violation(
                key='invalid.images.alt',
                value=result_no_alt,
                points=20 * len(result_no_alt)
            )

        if result_alt_too_big:
            self.add_violation(
                key='invalid.images.alt_too_big',
                value={
                    'images': result_alt_too_big,
                    'max_size': max_alt_size
                },
                points=20 * len(result_alt_too_big)
            )

    def get_imgs(self):
        return self.review.data.get('page.all_images', None)



#!/usr/bin/python
# -*- coding: utf-8 -*-


from holmes.validators.base import Validator
from holmes.utils import _


class TotalRequestsValidator(Validator):
    def validate(self):
        css_files = self.get_css_requests()
        js_files = self.get_js_requests()
        img_files = self.get_img_requests()

        self.add_fact(
            key='total.requests',
            value=len(css_files) + len(js_files) + len(img_files),
            title=_('Total requests')
        )

    def get_css_requests(self):
        return self.reviewer.current_html.cssselect('link[href]')

    def get_js_requests(self):
        return self.reviewer.current_html.cssselect('script[src]')

    def get_img_requests(self):
        return self.reviewer.current_html.cssselect('img[src]')



#!/usr/bin/python
# -*- coding: utf-8 -*-

from holmes.validators.base import Validator
from holmes.utils import _


class LinkWithRedirectValidator(Validator):

    @classmethod
    def get_violation_definitions(cls):
        return {
            'link.redirect.302': {
                'title': _('Link with 302 redirect'),
                'description': _(
                    'Link with 302 redirect, in most cases, should '
                    'not be used. Redirects were found for '
                    'link: %s.'),
                'category': _('HTTP'),
                'generic_description': _(
                    'Validates temporary redirections (302). '
                    'They should not be used in the most cases, instead '
                    'is best to use a 301 permanent redirect.'
                )
            },
            'link.redirect.307': {
                'title': _('Link with 307 redirect'),
                'description': _(
                    'Link with 307 redirect, in most cases, should '
                    'not be used. Redirects were found for '
                    'link: %s.'),
                'category': _('HTTP'),
                'generic_description': _(
                    'Validates temporary redirections (307). '
                    'They should not be used in the most cases, instead '
                    'is best to use a 301 permanent redirect.'
                )
            },
        }

    def validate(self):
        links = self.get_links()

        for url, response in links:
            if response.status_code in [302, 307]:
                self.add_violation(
                    key='link.redirect.%d' % response.status_code,
                    value=response.status_code,
                    points=10
                )

    def get_links(self):
        return self.review.data.get('page.links', None)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import sqlalchemy as sa
from sqlalchemy.orm import relationship
from sqlalchemy.orm.exc import NoResultFound

from holmes.models import Base


class Key(Base):
    __tablename__ = "keys"

    id = sa.Column(sa.Integer, primary_key=True)
    name = sa.Column('name', sa.String(2000), nullable=False)

    category_id = sa.Column(
        'category_id',
        sa.Integer,
        sa.ForeignKey('keys_category.id')
    )

    facts = relationship("Fact", cascade="all,delete", backref="key")
    violations = relationship("Violation", cascade="all,delete", backref="key")
    domains_violations_prefs = relationship("DomainsViolationsPrefs", cascade="all,delete", backref="key")
    users_violations_prefs = relationship("UsersViolationsPrefs", cascade="all,delete", backref="key")

    def __str__(self):
        return '%s' % (self.name)

    def __repr__(self):
        return str(self)

    def to_dict(self):
        return {
            'name': self.name,
            'category': self.category.name
        }

    @classmethod
    def get_by_name(cls, db, key_name):
        try:
            result = db.query(Key).filter(Key.name == key_name).one()
        except NoResultFound:
            result = None

        return result

    @classmethod
    def get_or_create(cls, db, key_name, category_name=None):
        key = cls.get_by_name(db, key_name)

        if key and key.category and key.category.name == category_name:
            return key

        if category_name is None:
            query_params = {'name': key_name}

            db.execute(
                'INSERT INTO `keys` (name) ' \
                'VALUES (:name) ON DUPLICATE KEY ' \
                'UPDATE name = :name',
                query_params
            )
        else:
            from holmes.models import KeysCategory
            category = KeysCategory.get_or_create(db, category_name)

            query_params = {'name': key_name, 'category_id': category.id}

            db.execute(
                'INSERT INTO `keys` (name, category_id) ' \
                'VALUES (:name, :category_id) ON DUPLICATE KEY ' \
                'UPDATE name = :name, category_id = :category_id',
                query_params
            )

        db.flush()

        return cls.get_by_name(db, key_name)

    @classmethod
    def insert_keys(cls, db, keys, default_values=[]):
        for name in keys.keys():
            category_name = keys[name].get('category', None)

            key = Key.get_or_create(db, name, category_name)
            keys[name]['key'] = key

            db.commit()

        for key in default_values:
            keys[key]['default_value'] = default_values[key].get('value', None)

            keys[key]['default_value_description'] = \
                default_values[key].get('description', None)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Deferred imports
try:
    from urllib.parse import urlparse  # Py3
except ImportError:
    from urlparse import urlparse

from holmes.utils import is_valid, _
from holmes.validators.base import Validator


class LinkWithRelCanonicalValidator(Validator):

    @classmethod
    def get_violation_definitions(cls):
        return {
            'absent.meta.canonical': {
                'title': _('Link with rel="canonical" not found'),
                'description': _(
                    'As can be seen in this page '
                    '<a href="https://support.google.com/webmasters/answer/'
                    '139394?hl=en">About rel="canonical"</a>, it\'s a good '
                    'practice to include rel="canonical" urls in the pages '
                    'for your website.'
                ),
                'category': _('SEO'),
                'generic_description': _(
                    'Validates the absent of link with rel="canonical" on '
                    'the head of a page. This indicates the preferred URL '
                    'to use to access the green dress post, so that the '
                    'search results will be more likely to show users '
                    'that URL structure.'
                ),
                'unit': 'boolean'
            }
        }

    @classmethod
    def get_default_violations_values(cls, config):
        return {
            'absent.meta.canonical': {
                'value': config.FORCE_CANONICAL,
                'description': config.get_description('FORCE_CANONICAL')
            }
        }

    def validate(self):
        force_canonical = self.get_violation_pref('absent.meta.canonical')

        if not force_canonical:
            # Only pages with query string parameters
            if self.page_url:
                if not is_valid(self.page_url):
                    return

                if not urlparse(self.page_url).query:
                    return

        head = self.get_head()
        if head:
            canonical = [item for item in head if item.get('rel') == 'canonical']

            if not canonical:
                self.add_violation(
                    key='absent.meta.canonical',
                    value=None,
                    points=30
                )

    def get_head(self):
        return self.review.data.get('page.head', None)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from holmes.search_providers import SearchProvider

from holmes.models.keys import Key
from holmes.models.page import Page
from holmes.models.violation import Violation

from pyelasticsearch import ElasticSearch
from pyelasticsearch.exceptions import (
    Timeout, ConnectionError, ElasticHttpError, InvalidJsonResponseError
)
from tornado.concurrent import return_future
from tornadoes import ESConnection
from ujson import loads, dumps
from datetime import datetime
from sqlalchemy import func
from sqlalchemy.orm import joinedload

import logging
import time


class ElasticSearchProvider(SearchProvider):
    def __init__(self, config, db=None, authnz_wrapper=None, io_loop=None):
        self.debug = False
        self.config = config
        if db is not None:
            self.db = db
        self.syncES = ElasticSearch(
            '%(ELASTIC_SEARCH_PROTOCOL)s://%(ELASTIC_SEARCH_HOST)s:%(ELASTIC_SEARCH_PORT)s' % config
        )
        self.asyncES = ESConnection(
            host=config.get('ELASTIC_SEARCH_HOST'),
            port=config.get('ELASTIC_SEARCH_PORT'),
            io_loop=io_loop,
            protocol=config.get('ELASTIC_SEARCH_PROTOCOL'),
        )
        self.index = config.get('ELASTIC_SEARCH_INDEX')
        self.max_retries = config.get('ELASTIC_SEARCH_MAX_RETRIES')

    def activate_debug(self):
        self.debug = True

    def connect_to_db(self):
        from sqlalchemy import create_engine
        from sqlalchemy.orm import scoped_session, sessionmaker
        conn_string = self.config.get('SQLALCHEMY_CONNECTION_STRING')
        engine = create_engine(
            conn_string,
            convert_unicode=True,
            pool_size=1,
            max_overflow=0,
            echo=self.debug
        )
        maker = sessionmaker(bind=engine, autoflush=True)
        self.db = scoped_session(maker)

    def _assemble_inner_query(self, domain=None, page_filter=None):
        if page_filter and domain:
            page_prefix = '%s/%s' % (domain.url, page_filter)
        else:
            page_prefix = None

        if page_prefix:
            return {
                'prefix': {
                    'page_url': page_prefix
                }
            }
        else:
            return {
                'match_all': {}
            }

    def _assemble_outer_query(self, inner_query, filter_terms):
        return {
            'filtered': {
                'query': inner_query,
                'filter': {
                    'and': [{
                        'term': filter_term
                    } for filter_term in filter_terms]
                }
            }
        }

    def _assemble_filter_terms(self, key_id=None, domain=None):
        filter_terms = []

        if key_id:
            filter_terms.append({'keys.id': key_id})

        if domain:
            filter_terms.append({'domain_id': domain.id})

        return filter_terms

    def gen_doc(self, review):
        return {
            'keys': [{'id': violation.key_id} for violation in review.violations],
            'uuid': str(review.uuid),
            'completed_date': review.completed_date,
            'violation_count': review.violation_count,
            'page_id': review.page_id,
            'page_uuid': str(review.page.uuid),
            'page_url': review.page.url,
            'page_last_review_date': review.page.last_review_date,
            'domain_id': review.domain_id,
            'domain_name': review.domain.name,
        }

    def index_review(self, review):
        for attempt in range(self.max_retries):
            try:
                self.syncES.send_request(
                    method='POST',
                    path_components=[self.index, 'review', review.page_id],
                    body=dumps(self.gen_doc(review)),
                    encode_body=False
                )
                break
            except (Timeout, ConnectionError, ElasticHttpError, InvalidJsonResponseError) as e:
                values = review.id, review.page_id, str(e)
                logging.error('Could not index review (review_id:{0}, page_id:{1}): {2}'.format(*values))
                time.sleep(1)
                if attempt >= self.max_retries - 1:
                    raise
            else:
                raise

    def index_reviews(self, reviewed_pages, reviews_count, batch_size):
        action = {'index': {'_type': 'review'}}

        for i in range(0, reviews_count, batch_size):
            body_bits = []

            for page in reviewed_pages[i:i + batch_size]:
                doc = self.gen_doc(page.last_review)

                action['index']['_id'] = doc['page_id']

                body_bits.append(dumps(action))
                body_bits.append(dumps(doc))

            # Yes, that trailing newline IS necessary
            body = '\n'.join(body_bits) + '\n'

            self.syncES.send_request(
                method='POST',
                path_components=[self.index, '_bulk'],
                body=body,
                encode_body=False
            )

        logging.info('Done!')

    @return_future
    def get_by_violation_key_name(self, key_id, current_page=1, page_size=10, domain=None, page_filter=None, callback=None):
        def treat_response(response):
            if response.error is None:
                try:
                    hits = loads(response.body).get('hits', {'hits': []})

                    reviews_data = []
                    for hit in hits['hits']:
                        completedAt = datetime.utcfromtimestamp(hit['_source']['completed_date'])
                        reviews_data.append({
                            'uuid': hit['_source']['uuid'],
                            'page': {
                                'uuid': hit['_source']['page_uuid'],
                                'url': hit['_source']['page_url'],
                                'completedAt': completedAt
                            },
                            'domain': hit['_source']['domain_name']
                        })

                    reviews_count = hits.get('total', 0)

                    callback({
                        'reviews': reviews_data,
                        'reviewsCount': reviews_count
                    })
                except Exception as e:
                    reason = 'ElasticSearchProvider: invalid response (%s [%s])' % (type(e), e.message)
                    logging.error(reason)
                    callback({'error': {'status_code': 500, 'reason': reason}})
            else:
                reason = 'ElasticSearchProvider: erroneous response (%s [%s])' % (response.error.message, response.body)
                logging.error(reason)
                callback({'error': {'status_code': 500, 'reason': reason}})

        inner_query = self._assemble_inner_query(domain, page_filter)
        filter_terms = self._assemble_filter_terms(key_id, domain)

        query = self._assemble_outer_query(inner_query, filter_terms)

        sort_ = [{
            'completed_date': {
                'order': 'desc'
            }
        }, {
            'violation_count': {
                'order': 'desc'
            }
        }]

        source = {'query': query, 'sort': sort_}

        self.asyncES.search(
            callback=treat_response,
            index=self.index,
            type='review',
            source=source,
            page=current_page,
            size=page_size,
        )

    @return_future
    def get_domain_active_reviews(self, domain, current_page=1, page_size=10, page_filter=None, callback=None):
        def treat_response(response):
            if response.error is None:
                try:
                    hits = loads(response.body).get('hits', {'hits': []})

                    pages = []
                    for hit in hits['hits']:
                        completedAt = datetime.utcfromtimestamp(hit['_source']['completed_date'])
                        pages.append({
                            'url': hit['_source']['page_url'],
                            'uuid': hit['_source']['page_uuid'],
                            'violationCount': len(hit['_source']['keys']),
                            'completedAt': completedAt,
                            'reviewId': hit['_source']['uuid']
                        })

                    reviews_count = hits.get('total', 0)

                    callback({
                        'reviewsCount': reviews_count,
                        'pages': pages
                    })
                except Exception as e:
                    reason = 'ElasticSearchProvider: invalid response (%s [%s])' % (type(e), e.message)
                    logging.error(reason)
                    callback({'error': {'status_code': 500, 'reason': reason}})
            else:
                reason = 'ElasticSearchProvider: erroneous response (%s [%s])' % (response.error.message, response.body)
                logging.error(reason)
                callback({'error': {'status_code': 500, 'reason': reason}})

        inner_query = self._assemble_inner_query(domain=domain, page_filter=page_filter)
        filter_terms = self._assemble_filter_terms(domain=domain)

        query = self._assemble_outer_query(inner_query, filter_terms)

        sort_ = [{
            'violation_count': {
                'order': 'desc'
            }
        }, {
            'completed_date': {
                'order': 'desc'
            }
        }]

        source = {'query': query, 'sort': sort_}

        self.asyncES.search(
            callback=treat_response,
            index=self.index,
            type='review',
            source=source,
            page=current_page,
            size=page_size,
        )

    def refresh(self):
        try:
            self.syncES.refresh(index=self.index)
        except Exception as e:
            logging.error('Could not refresh index (%s)' % e)

    def get_index_settings(cls):
        return {
            'index': {
                'number_of_shards': 4
            }
        }

    def get_index_mapping(cls):
        return {
            'review': {
                'properties': {
                    'keys': {
                        'properties': {
                            'id': {
                                'type': 'integer'
                            }
                        }
                    },
                    'uuid': {
                        'type': 'string',
                        'index': 'not_analyzed'
                    },
                    'completed_date': {
                        'type': 'integer'
                    },
                    'violation_count': {
                        'type': 'float'
                    },
                    'page_id': {
                        'type': 'integer'
                    },
                    'page_uuid': {
                        'type': 'string',
                        'index': 'not_analyzed'
                    },
                    'page_url': {
                        'type': 'string',
                        'index': 'not_analyzed'
                    },
                    'page_last_review_date': {
                        'type': 'integer'
                    },
                    'domain_id': {
                        'type': 'integer'
                    },
                    'domain_name': {
                        'type': 'string',
                        'index': 'not_analyzed'
                    }
                }
            }
        }

    def setup_index(self):
        try:
            settings = self.get_index_settings()
            self.syncES.create_index(index=self.index, settings=settings)
            mapping = self.get_index_mapping()
            self.syncES.put_mapping(index=self.index, doc_type='review', mapping=mapping)
            logging.info('Index %s created.' % self.index)
        except Exception as e:
            raise e

    def delete_index(self):
        try:
            self.syncES.delete_index(index=self.index)
            logging.info('Index %s deleted.' % self.index)
        except Exception as e:
            raise e

    def _get_max_page_id_from_index(self, must_have_domain_name=False):
        if must_have_domain_name:
            inner_query = {
                'constant_score': {
                    'filter': {
                        'not': {
                            'missing': {
                                'field': 'domain_name'
                            }
                        }
                    }
                }
            }
        else:
            inner_query = {
                'match_all': {}
            }

        query = {
            'query': inner_query,
            'sort': [{
                'page_id': {
                    'order': 'desc'
                }
            }]
        }

        results = self.syncES.search(query, index=self.index, doc_type='review')
        if results['hits']['total'] > 0:
            return results['hits']['hits'][0]['_id'] or 0
        return 0

    def index_all_reviews(self, keys=None, batch_size=200, replace=False):
        logging.info('Querying database...')
        self.connect_to_db()

        if keys is not None:
            keys = [k.id for k in self.db.query(Key.id).filter(Key.name.in_(keys)).all()]

        try:
            max_page_id = self._get_max_page_id_from_index(must_have_domain_name=True)
        except Exception:
            logging.error('Could not retrieve max page_id! Use with --replace (with caution)')
            return

        def apply_filters(query):
            if keys is not None:
                query = query \
                    .filter(Violation.review_id == Page.last_review_id) \
                    .filter(Violation.key_id.in_(keys))

            if not replace:
                query = query.filter(Page.id > max_page_id)

            return query.filter(Page.last_review_id != None)

        reviews_count = apply_filters(self.db.query(func.count(Page))).scalar()

        query = self.db.query(Page).options(joinedload('last_review'))
        reviewed_pages = apply_filters(query).order_by(Page.id.asc())

        logging.info('Indexing %d reviews...' % reviews_count)

        self.index_reviews(reviewed_pages, reviews_count, batch_size)

    @classmethod
    def new_instance(cls, config):
        return ElasticSearchProvider(config)

    @classmethod
    def main(cls):
        import sys

        parser = cls.argparser()
        args = parser.parse_args()

        config = {}
        host = None
        port = None
        index = None
        es = None

        levels = ['ERROR', 'WARNING', 'INFO', 'DEBUG']
        log_level = levels[args.verbose]
        logging.basicConfig(level=getattr(logging, log_level), format='%(levelname)s - %(message)s')

        if not (args.create or args.recreate or args.delete or args.keys or args.all_keys):
            parser.print_help()
            sys.exit(1)

        if args.conf:
            from derpconf.config import ConfigurationError
            from holmes.config import Config
            try:
                config = Config().load(args.conf[0])
                host = config['ELASTIC_SEARCH_HOST']
                port = config['ELASTIC_SEARCH_PORT']
                index = config['ELASTIC_SEARCH_INDEX']
            except ConfigurationError:
                logging.error('Could not load config! Use --conf conf_file')
                sys.exit(1)
            except KeyError:
                logging.error('Could not parse config! Check it\'s contents')
                sys.exit(1)

        if args.server:
            try:
                host, port = args.server[0].split(':')
                config['ELASTIC_SEARCH_HOST'] = host
                config['ELASTIC_SEARCH_PORT'] = port
            except Exception:
                logging.error('Could not parse server host and port! Use --server host:port')
                sys.exit(1)

        if args.index:
                index = args.index[0]
                config['ELASTIC_SEARCH_INDEX'] = index

        from pyelasticsearch.exceptions import IndexAlreadyExistsError, ElasticHttpNotFoundError, InvalidJsonResponseError
        from requests.exceptions import ConnectionError
        try:

            if args.create or args.recreate or args.delete:
                if host is None or port is None:
                    logging.error('Need either a host and port or a config file to perform such operation!')
                    sys.exit(1)
                if index is None:
                    logging.error('Need either an index name or a config file to perform such operation!')
                    sys.exit(1)
                else:
                    es = cls.new_instance(config)
                    if args.recreate or args.delete:
                        try:
                            es.delete_index()
                        except ElasticHttpNotFoundError:
                            pass
                        except InvalidJsonResponseError as e:
                            logging.error('Invalid response! Reason: %s' % e)
                            sys.exit(1)
                    if args.create or args.recreate:
                        es.setup_index()

            if args.keys or args.all_keys:
                if config is None:
                    logging.error('Need a config file to perform such operation! Use --conf conf_file')
                else:
                    batch_size = args.batch_size[0] if args.batch_size else 200
                    es = cls.new_instance(config) if not es else es
                    try:
                        if args.verbose > 2:
                            es.activate_debug()
                        if args.keys:
                            es.index_all_reviews(args.keys, replace=args.replace, batch_size=batch_size)
                        elif args.all_keys:
                            es.index_all_reviews(replace=args.replace, batch_size=batch_size)
                    except InvalidJsonResponseError as e:
                        logging.error('Invalid response! Reason: %s' % e)
                        sys.exit(1)

        except IndexAlreadyExistsError:
            logging.error('Index %s already exists! Use --recreate (with caution) to recreate' % index)
        except ConnectionError:
            logging.error('Could not connect to server at %s:%s' % (host, port))
        except KeyError:
            logging.error('Could not get host nor port! Use either -conf or --server')
            sys.exit(1)

if __name__ == '__main__':
    ElasticSearchProvider.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from tornado.gen import coroutine
from ujson import loads

from holmes.models import UsersViolationsPrefs
from holmes.handlers import BaseHandler


class UsersViolationsPrefsHandler(BaseHandler):

    @coroutine
    def get(self):
        user = self.get_authenticated_user()

        if not user:
            return

        user_prefs = UsersViolationsPrefs.get_prefs(self.db, user)

        violation_defs = self.application.violation_definitions

        user_prefs_keys = set(user_prefs.keys())
        violation_defs_keys = set(violation_defs.keys())

        insert_items = list(violation_defs_keys - user_prefs_keys)
        if insert_items:
            UsersViolationsPrefs.insert_prefs(self.db, user, insert_items)

        remove_items = list(user_prefs_keys - violation_defs_keys)
        if remove_items:
            UsersViolationsPrefs.delete_prefs(self.db, user, remove_items)

        result = []
        for key_name in violation_defs:
            category = violation_defs[key_name]['category']
            name = violation_defs[key_name]['title']
            description = violation_defs[key_name]['generic_description']
            is_active = user_prefs.get(key_name, True)

            result.append({
                'key': key_name,
                'name': self._(name),
                'description': self._(description),
                'category': self._(category),
                'is_active': is_active
            })

        self.write_json(result)

    @coroutine
    def post(self):
        user = self.get_authenticated_user()

        if not user:
            return

        post_data = loads(self.request.body)

        UsersViolationsPrefs.update_by_user(self.db, user, post_data)

        self.write_json({
            'reason': self._('Preferences successfully saved!'),
            'description': self._('Preferences successfully saved!')
        })



#!/usr/bin/python
# -*- coding: utf-8 -*-

from holmes.validators.base import Validator
from holmes.utils import _


class TitleValidator(Validator):

    @classmethod
    def get_violation_definitions(cls):
        return {
            'page.title.not_found': {
                'title': _('Page title not found'),
                'description': _("Title was not found on '%s'."),
                'category': _('HTTP'),
                'generic_description': _(
                    'Validates the presence of the page\'s title. '
                    'The <title> tag is required in all HTML documents '
                    'and it defines the title of the document.'
                )
            },
            'page.title.multiple': {
                'title': _('Too many titles'),
                'description': _(
                    "Page '%(page_url)s' has %(title_count)d title tags."),
                'category': _('Semantics'),
                'generic_description': _(
                    'Validates the presence of more than one page\'s title '
                    'tag.'
                )
            },
            'page.title.size': {
                'title': _('Maximum size of a page title'),
                'description': _(
                    'Title is too long on "%(page_url)s". '
                    'The max size is %(max_size)d characters.'),
                'category': _('SEO'),
                'generic_description': _(
                    'Validates the size of the page\'s title.'),
                'unit': 'number'
            }
        }

    @classmethod
    def get_default_violations_values(cls, config):
        return {
            'page.title.size': {
                'value': config.MAX_TITLE_SIZE,
                'description': config.get_description('MAX_TITLE_SIZE')
            }
        }

    def validate(self):
        max_title_size = self.get_violation_pref('page.title.size')

        title_count = self.review.data.get('page.title_count', 0)
        title = self.review.data.get('page.title', None)

        if not title_count or not title:
            self.add_violation(
                key='page.title.not_found',
                value=self.reviewer.page_url,
                points=50
            )
            return

        if title_count > 1:
            self.add_violation(
                key='page.title.multiple',
                value={
                    'page_url': self.reviewer.page_url,
                    'title_count': title_count
                },
                points=50
            )
        elif max_title_size and len(title) > int(max_title_size):
            self.add_violation(
                key='page.title.size',
                value={
                    'page_url': self.reviewer.page_url,
                    'max_size': int(max_title_size)
                },
                points=10
            )



#!/usr/bin/python
# -*- coding: utf-8 -*-

from holmes.validators.base import Validator
from holmes.utils import _


class RequiredMetaTagsValidator(Validator):

    @classmethod
    def get_violation_definitions(cls):
        return {
            'absent.meta.tags': {
                'title': _('Required Meta Tags were not found'),
                'description': _("Meta tags for %s were not found."),
                'value_parser': lambda value: ', '.join(value),
                'category': _('HTTP'),
                'generic_description': _(
                    'Validates the absent of user defined MetaTags. '
                    'This values are configurable by Holmes Configuration.'
                ),
                'unit': 'list'
            }
        }

    @classmethod
    def get_default_violations_values(cls, config):
        return {
            'absent.meta.tags': {
                'value': config.REQUIRED_META_TAGS,
                'description': config.get_description('REQUIRED_META_TAGS')
            }
        }

    def validate(self):
        required_tags = self.get_violation_pref('absent.meta.tags')

        absent_metatags = []

        for tag in required_tags:
            if not self.get_meta_tag(tag):
                absent_metatags.append(tag)

        if absent_metatags:
            self.add_violation(
                key='absent.meta.tags',
                value=absent_metatags,
                points=20
            )

    def get_meta_tag(self, tag):
        meta_tags = self.review.data.get('meta.tags', None)
        return [i for i in meta_tags if i['key'] == tag]



#!/usr/bin/python
# -*- coding: utf-8 -*-

from holmes.facters import Facter
from holmes.utils import _


class TitleFacter(Facter):

    @classmethod
    def get_fact_definitions(cls):
        return {
            'page.title': {
                'title': _('Page Title'),
                'description': lambda value: value,
                'category': _('SEO'),
            }
        }

    def get_facts(self):
        titles = self.reviewer.current_html.cssselect('title')

        if not titles:
            return

        if not titles[0].text:
            return

        self.review.data['page.title_count'] = len(titles)

        title = titles[0].text.strip()

        self.review.data['page.title'] = title
        self.add_fact(key='page.title', value=title)



"""create delimiter table

Revision ID: 43ce6999f48e
Revises: 56e5b86b4b40
Create Date: 2014-02-10 18:27:15.111090

"""

# revision identifiers, used by Alembic.
revision = '43ce6999f48e'
down_revision = '37881a97d680'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_table(
        'delimiters',
        sa.Column('id', sa.Integer, primary_key=True),
        sa.Column('url', sa.String(2000), nullable=False),
        sa.Column('url_hash', sa.String(128), nullable=False),
        sa.Column('value', sa.Integer, nullable=False, server_default='1')
    )

    op.create_index('idx_url_hash', 'delimiters', ['url_hash'])


def downgrade():
    op.drop_index('idx_url_hash', 'delimiters')
    op.drop_table('delimiters')




"""create violation table

Revision ID: 3283f8a20fc9
Revises: 52988f72b077
Create Date: 2013-11-13 11:36:56.729207

"""

# revision identifiers, used by Alembic.
revision = '3283f8a20fc9'
down_revision = '52988f72b077'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_table(
        'violations',
        sa.Column('id', sa.Integer, primary_key=True),
        sa.Column('title', sa.String(2000), nullable=False),
        sa.Column('description', sa.String(2000), nullable=False),
        sa.Column('key', sa.String(2000), nullable=False),
        sa.Column('points', sa.Integer, nullable=False),
        sa.Column('review_id', sa.Integer),
    )

    op.create_foreign_key(
        "fk_violation_review", "violations",
        "reviews", ["review_id"], ["id"]
    )


def downgrade():
    op.drop_constraint('fk_violation_review', 'violations', type_="foreignkey")
    op.drop_table('violations')



#!/usr/bin/python
# -*- coding: utf-8 -*-

from tornado.gen import coroutine
from tornado import gen
from ujson import loads

from holmes.models import Domain, DomainsViolationsPrefs
from holmes.handlers import BaseHandler


class DomainsViolationsPrefsHandler(BaseHandler):

    @coroutine
    def options(self, domain_name=None):
        super(DomainsViolationsPrefsHandler, self).options()

    @coroutine
    def get(self, domain_name):
        domain = Domain.get_domain_by_name(domain_name, self.db)

        if not domain:
            self.set_status(404, self._('Domain %s not found') % domain_name)
            return

        prefs = DomainsViolationsPrefs.get_domains_violations_prefs_by_domain(
            self.db, domain.name
        )

        violation_defs = self.application.violation_definitions

        result = []

        for pref in prefs:
            key = violation_defs.get(pref.get('key'))

            if key is None:
                continue

            result.append({
                'key': pref.get('key'),
                'title': key.get('default_value_description', None),
                'category': key.get('category', None),
                'value': pref.get('value'),
                'default_value': key.get('default_value', None),
                'unit': key.get('unit', None)
            })

        self.write_json(result)

    @gen.coroutine
    def post(self, domain_name):
        if not self.validate_superuser():
            return

        domain = Domain.get_domain_by_name(domain_name, self.db)

        if not domain:
            self.set_status(404, self._('Domain %s not found') % domain_name)
            return

        post_data = loads(self.request.body)

        DomainsViolationsPrefs.update_by_domain(
            self.db, self.cache, domain, post_data
        )

        self.write_json({
            'reason': self._('Preferences successfully saved!'),
            'description': self._('Preferences successfully saved!')
        })



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from holmes.search_providers.elastic import ElasticSearchProvider
from holmes.utils import load_classes


class AuthNZElasticSearchProvider(ElasticSearchProvider):
    def __init__(self, config, db=None, authnz_wrapper=None, io_loop=None):

        if not authnz_wrapper:
            raise Exception('An authentication/authorization wrapper must be defined!')

        super(AuthNZElasticSearchProvider, self).__init__(config, db, io_loop)

        self.syncES.session = authnz_wrapper.sync_client

        self.asyncES.client = authnz_wrapper.async_client

    @classmethod
    def new_instance(cls, config):
        authnz_wrapper_list = load_classes(default=[config.AUTHNZ_WRAPPER])
        if isinstance(authnz_wrapper_list, list) and len(authnz_wrapper_list) == 1:
            authnz_wrapper_class = authnz_wrapper_list.pop()
        else:
            raise Exception('An authentication/authorization wrapper must be defined!')

        authnz_wrapper = authnz_wrapper_class(config)

        return AuthNZElasticSearchProvider(
            config=config,
            authnz_wrapper=authnz_wrapper
        )


if __name__ == '__main__':
    AuthNZElasticSearchProvider.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from holmes.validators.base import Validator
from holmes.utils import _


class GoogleAnalyticsValidator(Validator):

    @classmethod
    def get_violation_definitions(cls):
        return {
            'google_analytics.not_found': {
                'title': _('Google Analytics not found'),
                'description': _(
                    'This page should include a Google Analytics.'),
                'category': _('SEO'),
                'generic_description': _(
                    'Google Analytics script is important to measure '
                    'various kinds of statistics of the page. It\'s a SEO '
                    'violation an ausence of this script.'
                )
            },
            'google_analytics.account.not_found': {
                'title': _('Google Analytics account not found'),
                'description': _(
                    'This page should include a Google Analytics account.'),
                'category': _('SEO'),
                'generic_description': _(
                    'Google Analytics script is important to measure '
                    'various kinds of statistics of the page. The '
                    'configuration of the script needs an account '
                    'to log in. Not founding this information may '
                    'cause errors to log the statistics in the Google API.'
                )
            },
            'google_analytics.domain.not_found': {
                'title': _('Google Analytics domain not found'),
                'description': _(
                    'This page should include a Google Analytics domain.'),
                'category': _('SEO'),
                'generic_description': _(
                    'Google Analytics script is important to measure '
                    'various kinds of statistics of the page. The '
                    'configuration of the script needs a domain '
                    'information to work well.'
                )
            }
        }

    def validate(self):
        google_analytics = self.get_analytics()

        if not google_analytics:
            self.add_violation(
                key='google_analytics.not_found',
                value=None,
                points=100
            )
            return

        for account, domain in google_analytics:
            if not account:
                self.add_violation(
                    key='google_analytics.account.not_found',
                    value=None,
                    points=50
                )

            if not domain:
                self.add_violation(
                    key='google_analytics.domain.not_found',
                    value=None,
                    points=50
                )

    def get_analytics(self):
        return self.review.data.get('page.google_analytics', None)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import re

from holmes.facters import Facter
from holmes.utils import _


class GoogleAnalyticsFacter(Facter):

    @classmethod
    def get_fact_definitions(cls):
        return {
            'page.google_analytics': {
                'title': _('Google Analytics'),
                'description': lambda value: list(value),
                'unit': 'google-analytics',
                'category': _('SEO'),
            }
        }

    def get_facts(self):
        self.review.data['page.google_analytics'] = set()

        script_data = self.get_script_data()

        account_regex = "_?setAccount[^,]*,[^']*'([^']*)'"
        domain_regex = "-?setDomainName[^,]*,[^']*'([^']*)'"

        analytics_re = re.compile(
            r"%s(.*?)%s" % (account_regex, domain_regex),
            re.MULTILINE | re.DOTALL)

        for i in script_data:
            if i.text:
                analytics_data = analytics_re.findall(i.text)
                for account, _, domain in analytics_data:
                    self.review.data['page.google_analytics'].add(
                        (account, domain)
                    )

        if self.review.data['page.google_analytics']:
            self.add_fact(
                key='page.google_analytics',
                value=self.review.data['page.google_analytics'],
            )

    def get_script_data(self):
        return self.reviewer.current_html.cssselect('script')



#!/usr/bin/python
# -*- coding: utf-8 -*-

from holmes.validators.base import Validator
from holmes.facters.links import REMOVE_HASH
from holmes.utils import _


class AnchorWithoutAnyTextValidator(Validator):
    @classmethod
    def get_empty_anchors_parsed_value(cls, value):
        return ', '.join([
            '<a href="%s" target="_blank">#%s</a>' % (href, index)
            for index, href in enumerate(value)
        ])

    @classmethod
    def get_violation_definitions(cls):
        return {
            'empty.anchors': {
                'title': _('Empty anchor(s) found'),
                'description': _(
                    'Empty anchors are not good for Search Engines. '
                    'Empty anchors were found for links to: %s.'),
                'value_parser': cls.get_empty_anchors_parsed_value,
                'category': _('SEO'),
                'generic_description': _(
                    'By using empty anchor text won\'t prevent search '
                    'engines from indexing your pages but you will lose a '
                    'good opportunity to add relevance to your pages. '
                    'Google uses anchor text in order to qualify the '
                    'resources you create a reference to. '
                    'In consequences if a page got links with "example" '
                    'as anchor text pointing to itself, this page '
                    'relevance on "example" request will increase. '
                    'So better do not let empty anchor text and choose '
                    'wisely the words (or keywords) you use in it.')
            }
        }

    def validate(self):
        links = self.get_links()

        links_with_empty_anchor = []

        for link in links:
            href = link.get('href').strip()
            href = REMOVE_HASH.sub('', href)

            if href and not link.text_content() and not link.findall('img'):
                href = self.normalize_url(href)
                if not href:
                    continue
                links_with_empty_anchor.append(href)

        if links_with_empty_anchor:
            self.add_violation(
                key='empty.anchors',
                value=links_with_empty_anchor,
                points=20 * len(links_with_empty_anchor)
            )

    def get_links(self):
        return self.review.data.get('page.all_links', None)



"""Create users_violations_prefs table

Revision ID: 47fac4be8ca7
Revises: 19ffab346000
Create Date: 2014-07-14 19:00:15.740884

"""

# revision identifiers, used by Alembic.
revision = '47fac4be8ca7'
down_revision = '19ffab346000'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_table(
        'users_violations_prefs',
        sa.Column('id', sa.Integer, primary_key=True),
        sa.Column('user_id', sa.Integer, nullable=False),
        sa.Column('key_id', sa.Integer, nullable=False),
        sa.Column('is_active', sa.Boolean, default=True, nullable=False),
    )

    op.create_unique_constraint(
        'uk_user_key',
        'users_violations_prefs',
        ['key_id', 'user_id']
    )

    op.create_foreign_key(
        'fk_user', 'users_violations_prefs',
        'users', ['user_id'], ['id']
    )

    op.create_foreign_key(
        'fk_key_id', 'users_violations_prefs',
        'keys', ['key_id'], ['id']
    )


def downgrade():
    op.drop_constraint('fk_user', 'users_violations_prefs', type_='foreignkey')
    op.drop_constraint('fk_key_id', 'users_violations_prefs', type_='foreignkey')
    op.drop_constraint('uk_user_key', 'users_violations_prefs', type_='unique')
    op.drop_table('users_violations_prefs')



"""Domain: Created name index

Revision ID: 2e3c3d79c318
Revises: 5592365e4624
Create Date: 2014-12-03 14:39:47.626645

"""

# revision identifiers, used by Alembic.
revision = '2e3c3d79c318'
down_revision = '5592365e4624'

from alembic import op


def upgrade():
    op.execute('CREATE INDEX idx_name on domains (name(220))')


def downgrade():
    op.drop_index('idx_name', 'domains')



#!/usr/bin/python
# -*- coding: utf-8 -*-

import re

from holmes.validators.base import Validator
from holmes.utils import get_domain_from_url, _

REMOVE_HASH = re.compile('([#].*)$')


class LinkCrawlerValidator(Validator):
    def __init__(self, *args, **kw):
        super(LinkCrawlerValidator, self).__init__(*args, **kw)
        self.broken_links = set()
        self.moved_links = set()

    @classmethod
    def get_links_parsed_value(cls, value):
        return {'links': ', '.join([
            '<a href="%s" target="_blank">Link #%s</a>' % (url, index)
            for index, url in enumerate(value)
        ])}

    @classmethod
    def get_violation_definitions(cls):
        return {
            'link.broken': {
                'title': _('Broken link(s) found'),
                'description': _(
                    'This page contains broken links to %(links)s '
                    '(the urls failed to load or timed-out after 10 seconds). '
                    'This can lead your site to lose rating with '
                    'search engines and is misleading to users.'),
                'value_parser': cls.get_links_parsed_value,
                'category': _('HTTP'),
                'generic_description': _(
                    'Validates hyperlinks with a invalid resource. '
                    'The pointed resource can be a Client or a '
                    'Server Error, they can be timeout load or, '
                    'in most cases, means a not founded page.'
                )
            },
            'link.moved.temporarily': {
                'title': _('Moved Temporarily'),
                'description': _(
                    'A link from your page to "%(links)s" is using a 302 or '
                    '307 redirect. It passes 0%% of link juice (ranking '
                    'power) and, in most cases, should not be used. Use '
                    '301 instead.'),
                'value_parser': cls.get_links_parsed_value,
                'category': _('HTTP'),
                'generic_description': _(
                    'Validates links that uses a 302 or 307 redirect. '
                    'In this cases, it passes 0% of link juice (ranking '
                    'power) and, in most cases, should not be used. Use of '
                    '301 is recommended.'
                )
            }
        }

    def validate(self):
        links = self.get_links()

        total_score = float(self.reviewer.page_score)
        tax = total_score * float(self.reviewer.config.PAGE_SCORE_TAX_RATE)
        available_score = total_score - tax

        number_of_links = float(len(links)) or 1.0
        link_score = available_score / number_of_links

        for url, response in links:
            domain, domain_url = get_domain_from_url(url)
            if domain in self.page_url:
                self.send_url(response.effective_url, link_score, response)
            else:
                if response.status_code in [302, 307]:
                    self.moved_link_violation(url, response)
                elif response.status_code > 399:
                    self.broken_link_violation(url, response)

        if self.broken_links:
            self.add_violation(
                key='link.broken',
                value=self.broken_links,
                points=100 * len(self.broken_links)
            )

        if self.moved_links:
            self.add_violation(
                key='link.moved.temporarily',
                value=self.moved_links,
                points=100
            )

        self.flush()

    def flush(self, *args, **kw):
        super(LinkCrawlerValidator, self).flush(*args, **kw)
        self.broken_links = set()
        self.moved_links = set()

    def get_links(self):
        return self.review.data['page.links']

    def broken_link_violation(self, url, response):
        self.broken_links.add(url)

    def moved_link_violation(self, url, response):
        self.moved_links.add(url)



"""Changing violations value to Longtext

Revision ID: 368929fa3075
Revises: 54ca0bd9bc1e
Create Date: 2014-01-10 14:01:54.876125

"""

# revision identifiers, used by Alembic.
revision = '368929fa3075'
down_revision = '54ca0bd9bc1e'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.alter_column('violations', 'value', type_=sa.Text(length=4294967295), nullable=True)


def downgrade():
    op.alter_column('violations', 'value', type_=sa.Text(), nullable=True)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import sqlalchemy as sa
from sqlalchemy.orm import relationship

from holmes.models import Base


class User(Base):
    __tablename__ = "users"

    id = sa.Column(sa.Integer, primary_key=True)
    fullname = sa.Column('fullname', sa.String(200), nullable=False)
    email = sa.Column('email', sa.String(100), nullable=False, unique=True)
    provider = sa.Column('provider', sa.String(10), nullable=True)
    is_superuser = sa.Column(
        'is_superuser',
        sa.Boolean,
        nullable=False,
        default=False
    )
    last_login = sa.Column('last_login', sa.DateTime, nullable=True)
    locale = sa.Column('locale', sa.String(10), nullable=True)

    violations_prefs = relationship('UsersViolationsPrefs', backref='user')

    def to_dict(self):
        return {
            'fullname': self.fullname,
            'email': self.email,
            'is_superuser': bool(self.is_superuser),
            'last_login': self.last_login,
            'locale': self.locale,
        }

    def __str__(self):
        return str(self.email)

    def __repr__(self):
        return str(self)

    @classmethod
    def by_email(cls, email, db):
        return db.query(User).filter(User.email == email).first()

    @classmethod
    def add_user(cls, db, fullname, email, provider, last_login=None):
        user = User(fullname=fullname, email=email,
                    provider=provider, last_login=last_login)
        db.add(user)
        db.flush()
        return user

    @classmethod
    def update_locale(cls, db, user, locale):
        db \
            .query(User).filter(User.id == user.id) \
            .update({'locale': locale})



"""Allowing null facts and violations

Revision ID: 4ff6c7ad84c6
Revises: 49a08fc2113a
Create Date: 2013-11-27 21:27:50.757008

"""

# revision identifiers, used by Alembic.
revision = '4ff6c7ad84c6'
down_revision = '49a08fc2113a'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.alter_column('facts', 'value', type_=sa.Text, nullable=True)


def downgrade():
    op.alter_column('facts', 'value', type_=sa.Text, nullable=False)



#!/usr/bin/python
# -*- coding: utf-8 -*-

from holmes.validators.base import Validator
from holmes.utils import _


class JSRequestsValidator(Validator):

    @classmethod
    def get_violation_definitions(cls):
        return {
            'total.requests.js': {
                'title': _('Too many JavaScript requests'),
                'description': _(
                    'This page has %(total_js_files)d JavaScript request '
                    '(%(over_limit)d over limit). Having too many requests '
                    'impose a tax in the browser due to handshakes.'),
                'category': _('Performance'),
                'generic_description': _(
                    'A site with too many JavaScript requests per page can '
                    'deacrease the page load speed and performance. This '
                    'limits are configurable in Holmes configuration.'
                ),
                'unit': 'number'
            },
            'total.size.js': {
                'title': _('JavaScript size in kb is too big'),
                'description': _(
                    'There\'s %.2fkb of JavaScript in this page and '
                    'that adds up to more download time slowing down '
                    'the page rendering to the user.'),
                'category': _('Performance'),
                'generic_description': _(
                    'A site with a too big total JavaScript size per page can '
                    'decrease the page load speed and performance. This '
                    'limits are configurable in Holmes configuration.'
                ),
                'unit': 'number'
            }
        }

    @classmethod
    def get_default_violations_values(cls, config):
        return {
            'total.size.js': {
                'value': config.MAX_JS_KB_PER_PAGE_AFTER_GZIP,
                'description': config.get_description('MAX_JS_KB_PER_PAGE_AFTER_GZIP')
            },
            'total.requests.js': {
                'value': config.MAX_JS_REQUESTS_PER_PAGE,
                'description': config.get_description('MAX_JS_REQUESTS_PER_PAGE')
            }
        }

    def validate(self):
        max_size_js = self.get_violation_pref('total.size.js')

        max_requests_js = self.get_violation_pref('total.requests.js')

        total_js_files = self.get_total_requests_js()
        total_size_gzip = self.get_total_size_js_gzipped()

        over_limit = total_js_files - max_requests_js

        if total_js_files > max_requests_js:
            self.add_violation(
                key='total.requests.js',
                value={
                    'total_js_files': total_js_files,
                    'over_limit': over_limit},
                points=5 * over_limit
            )

        if total_size_gzip > max_size_js:
            self.add_violation(
                key='total.size.js',
                value=total_size_gzip,
                points=int(total_size_gzip - max_size_js)
            )

    def get_js_requests(self):
        return self.review.data.get('page.js', None)

    def get_total_requests_js(self):
        return self.review.data.get('total.requests.js', 0)

    def get_total_size_js(self):
        return self.review.data.get('total.size.js', 0)

    def get_total_size_js_gzipped(self):
        return self.review.data.get('total.size.js.gzipped', 0)



#!/usr/bin/python
# -*- coding: utf-8 -*-

from holmes.validators.base import Validator
from holmes.utils import _


class MetaTagsValidator(Validator):

    @classmethod
    def get_violation_definitions(cls):
        return {
            'absent.metatags': {
                'title': _('Meta tags not present'),
                'description': _(
                    'No meta tags found on this page. This is damaging for '
                    'Search Engines.'
                ),
                'category': _('HTTP'),
                'generic_description': _(
                    'Validates the presence of metatags. They are important '
                    'to inform metadata about the HTML document. '
                    'The absent of metatags are damaging for search '
                    'engines. Meta elements are typically used to specify '
                    'page description, keywords, author of the document, last '
                    'modified, and other metadata.'
                )
            },
            'page.metatags.description_too_big': {
                'title': _('Maximum size of description meta tag'),
                'description': _(
                    'The meta description tag is longer than %(max_size)s '
                    'characters. It is best to keep meta descriptions '
                    'shorter for better indexing on search engines.'
                ),
                'category': _('SEO'),
                'generic_description': _(
                    'Validates the size of a description metatag. It is best '
                    'to keep meta descriptions shorter for better indexing on '
                    'search engines. This limit is configurable by Holmes '
                    'Configuration.'
                ),
                'unit': 'number'
            }
        }

    @classmethod
    def get_default_violations_values(cls, config):
        return {
            'page.metatags.description_too_big': {
                'value': config.METATAG_DESCRIPTION_MAX_SIZE,
                'description': config.get_description('METATAG_DESCRIPTION_MAX_SIZE')
            }
        }

    def validate(self):
        max_size = self.get_violation_pref('page.metatags.description_too_big')

        meta_tags = self.review.data.get('meta.tags', None)

        if not meta_tags:
            self.add_violation(
                key='absent.metatags',
                value='No metatags.',
                points=100
            )

        for mt in meta_tags:
            if mt['key'] == 'description' and len(mt['content']) > max_size:
                self.add_violation(
                    key='page.metatags.description_too_big',
                    value={'max_size': max_size},
                    points=20
                )
                break



"""create user table

Revision ID: 37881a97d680
Revises: d8f500d9168
Create Date: 2014-02-10 15:52:50.366173

"""

# revision identifiers, used by Alembic.
revision = '37881a97d680'
down_revision = 'd8f500d9168'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_table(
        'users',
        sa.Column('id', sa.Integer, primary_key=True),
        sa.Column('fullname', sa.String(200), nullable=False),
        sa.Column('email', sa.String(100), nullable=False),
        sa.Column('last_login', sa.DateTime, nullable=True),
        sa.Column(
            'is_superuser',
            sa.Boolean,
            nullable=False,
            server_default='0'
        )
    )

    op.create_index('idx_email', 'users', ['email'])


def downgrade():
    op.drop_index('idx_email', 'users')
    op.drop_table('users')
